@@ -1,10 +1,10 @@
 /*
- * $Header: /cvsroot/pgsql/contrib/pgbench/pgbench.c,v 1.22.2.1 2007/01/13 03:31:12 ishii Exp $
+ * $Header: /cvsroot/pgsql-server/contrib/pgbench/pgbench.c,v 1.27.2.2 2003/12/07 19:56:07 tgl Exp $
  *
  * pgbench: a simple TPC-B like benchmark program for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2000-2007	Tatsuo Ishii
+ * Copyright (c) 2000-2002	Tatsuo Ishii
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -41,6 +41,10 @@
 #include <sys/resource.h>
 #endif   /* ! WIN32 */
 
+extern char *optarg;
+extern int	optind;
+
+
 /********************************************************************
  * some configurable parameters */
 
@@ -118,7 +122,7 @@ doConnect()
 {
 	PGconn	   *con;
 	PGresult   *res;
- 
+
 	con = PQsetdbLogin(pghost, pgport, pgoptions, pgtty, dbName,
 					   login, pwd);
 	if (con == NULL)
@@ -147,13 +151,6 @@ doConnect()
 		exit(1);
 	}
 	PQclear(res);
-	res = PQexec(con, "SET autocommit TO 'on'");
-	if (PQresultStatus(res) != PGRES_COMMAND_OK)
-	{
-		fprintf(stderr, "%s", PQerrorMessage(con));
-		exit(1);
-	}
-	PQclear(res);
 
 	return (con);
 }
@@ -264,14 +261,14 @@ doOne(CState * state, int n, int debug, int ttype)
 				 */
 				if (use_log)
 				{
-					long long	diff;
+					double	diff;
 					struct timeval now;
 
 					gettimeofday(&now, 0);
-					diff = (now.tv_sec - st->txn_begin.tv_sec) * 1000000 +
-						(now.tv_usec - st->txn_begin.tv_usec);
+					diff = (int) (now.tv_sec - st->txn_begin.tv_sec) * 1000000.0 +
+						(int) (now.tv_usec - st->txn_begin.tv_usec);
 
-					fprintf(LOGFILE, "%d %d %lld\n", st->id, st->cnt, diff);
+					fprintf(LOGFILE, "%d %d %.0f\n", st->id, st->cnt, diff);
 				}
 
 				res = PQgetResult(st->con);
@@ -485,13 +482,19 @@ init(void)
 	PGresult   *res;
 	static char *DDLs[] = {
 		"drop table branches",
-		"create table branches(bid int, primary key(bid),bbalance int,filler char(88))",
+		"create table branches(bid int not null,bbalance int,filler char(88))",
 		"drop table tellers",
-		"create table tellers(tid int, primary key(tid),bid int,tbalance int,filler char(84))",
+		"create table tellers(tid int not null,bid int,tbalance int,filler char(84))",
 		"drop table accounts",
-		"create table accounts(aid int,primary key(aid),bid int,abalance int,filler char(84))",
+		"create table accounts(aid int not null,bid int,abalance int,filler char(84))",
 		"drop table history",
 	"create table history(tid int,bid int,aid int,delta int,mtime timestamp,filler char(22))"};
+	static char *DDLAFTERs[] = {
+		"alter table branches add primary key (bid)",
+		"alter table tellers add primary key (tid)",
+		"alter table accounts add primary key (aid)"};
+
+
 	char		sql[256];
 
 	int			i;
@@ -570,7 +573,7 @@ init(void)
 			PQclear(res);
 		}
 
-		snprintf(sql, 256, "%d\t%d\t%d\t\n", j, j / naccounts, 0);
+		snprintf(sql, 256, "%d\t%d\t%d\t\n", j, i / naccounts + 1, 0);
 		if (PQputline(con, sql))
 		{
 			fprintf(stderr, "PQputline failed\n");
@@ -611,6 +614,17 @@ init(void)
 #endif   /* NOT_USED */
 		}
 	}
+	fprintf(stderr, "set primary key...\n");
+	for (i = 0; i < (sizeof(DDLAFTERs) / sizeof(char *)); i++)
+	{
+		res = PQexec(con, DDLAFTERs[i]);
+		if (strncmp(DDLs[i], "drop", 4) && PQresultStatus(res) != PGRES_COMMAND_OK)
+		{
+			fprintf(stderr, "%s", PQerrorMessage(con));
+			exit(1);
+		}
+		PQclear(res);
+	}
 
 	/* vacuum */
 	fprintf(stderr, "vacuum...");
@@ -668,10 +682,6 @@ printResults(
 int
 main(int argc, char **argv)
 {
-	extern char *optarg;
-	extern int	optind,
-				opterr,
-				optopt;
 	int			c;
 	int			is_init_mode = 0;		/* initialize mode? */
 	int			is_no_vacuum = 0;		/* no vacuum at all before
@@ -701,6 +711,14 @@ main(int argc, char **argv)
 
 	PGconn	   *con;
 	PGresult   *res;
+	char	   *env;
+
+	if ((env = getenv("PGHOST")) != NULL && *env != '\0')
+		pghost = env;
+	if ((env = getenv("PGPORT")) != NULL && *env != '\0')
+		pgport = env;
+	else if ((env = getenv("PGUSER")) != NULL && *env != '\0')
+		login = env;
 
 	while ((c = getopt(argc, argv, "ih:nvp:dc:t:s:U:P:CNSl")) != -1)
 	{
@@ -795,8 +813,11 @@ main(int argc, char **argv)
 		dbName = argv[optind];
 	else
 	{
-		dbName = getenv("USER");
-		if (dbName == NULL)
+		if ((env = getenv("PGDATABASE")) != NULL && *env != '\0')
+			dbName = env;
+		else if (login != NULL && *login != '\0')
+			dbName = login;
+		else
 			dbName = "";
 	}
 
@@ -809,7 +830,7 @@ main(int argc, char **argv)
 	remains = nclients;
 
 	state = (CState *) malloc(sizeof(*state) * nclients);
-	memset(state, 0, sizeof(*state));
+	memset(state, 0, sizeof(*state) * nclients);
 
 	if (use_log)
 	{
@@ -914,7 +935,7 @@ main(int argc, char **argv)
 
 	/* set random seed */
 	gettimeofday(&tv1, 0);
-	srandom((uint) tv1.tv_usec);
+	srand((uint) tv1.tv_usec);
 
 	/* get start up time */
 	gettimeofday(&tv1, 0);
