@@ -4,12 +4,12 @@
  *
  *	  Routines for CREATE and DROP FUNCTION commands
  *
- * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/commands/functioncmds.c,v 1.24.2.1 2003/02/01 22:09:41 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/commands/functioncmds.c,v 1.38.2.1 2004/02/21 00:35:13 tgl Exp $
  *
  * DESCRIPTION
  *	  These routines take the parse tree and pick out the
@@ -78,11 +78,15 @@ compute_return_type(TypeName *returnType, Oid languageOid,
 		if (!get_typisdefined(rettype))
 		{
 			if (languageOid == SQLlanguageId)
-				elog(ERROR, "SQL function cannot return shell type \"%s\"",
-					 TypeNameToString(returnType));
+				ereport(ERROR,
+						(errcode(ERRCODE_INVALID_FUNCTION_DEFINITION),
+					   errmsg("SQL function cannot return shell type %s",
+							  TypeNameToString(returnType))));
 			else
-				elog(NOTICE, "Return type \"%s\" is only a shell",
-					 TypeNameToString(returnType));
+				ereport(NOTICE,
+						(errcode(ERRCODE_WRONG_OBJECT_TYPE),
+						 errmsg("return type %s is only a shell",
+								TypeNameToString(returnType))));
 		}
 	}
 	else
@@ -100,20 +104,24 @@ compute_return_type(TypeName *returnType, Oid languageOid,
 		 */
 		if (languageOid != INTERNALlanguageId &&
 			languageOid != ClanguageId)
-			elog(ERROR, "Type \"%s\" does not exist", typnam);
+			ereport(ERROR,
+					(errcode(ERRCODE_UNDEFINED_OBJECT),
+					 errmsg("type \"%s\" does not exist", typnam)));
 
 		/* Otherwise, go ahead and make a shell type */
-		elog(NOTICE, "ProcedureCreate: type %s is not yet defined",
-			 typnam);
+		ereport(NOTICE,
+				(errcode(ERRCODE_UNDEFINED_OBJECT),
+				 errmsg("type \"%s\" is not yet defined", typnam),
+				 errdetail("Creating a shell type definition.")));
 		namespaceId = QualifiedNameGetCreationNamespace(returnType->names,
 														&typname);
 		aclresult = pg_namespace_aclcheck(namespaceId, GetUserId(),
 										  ACL_CREATE);
 		if (aclresult != ACLCHECK_OK)
-			aclcheck_error(aclresult, get_namespace_name(namespaceId));
+			aclcheck_error(aclresult, ACL_KIND_NAMESPACE,
+						   get_namespace_name(namespaceId));
 		rettype = TypeShellMake(typname, namespaceId);
-		if (!OidIsValid(rettype))
-			elog(ERROR, "could not create type %s", typnam);
+		Assert(OidIsValid(rettype));
 	}
 
 	*prorettype_p = rettype;
@@ -137,8 +145,10 @@ compute_parameter_types(List *argTypes, Oid languageOid,
 		Oid			toid;
 
 		if (parameterCount >= FUNC_MAX_ARGS)
-			elog(ERROR, "functions cannot have more than %d arguments",
-				 FUNC_MAX_ARGS);
+			ereport(ERROR,
+					(errcode(ERRCODE_TOO_MANY_ARGUMENTS),
+				   errmsg("functions cannot have more than %d arguments",
+						  FUNC_MAX_ARGS)));
 
 		toid = LookupTypeName(t);
 		if (OidIsValid(toid))
@@ -147,21 +157,29 @@ compute_parameter_types(List *argTypes, Oid languageOid,
 			{
 				/* As above, hard error if language is SQL */
 				if (languageOid == SQLlanguageId)
-					elog(ERROR, "SQL function cannot accept shell type \"%s\"",
-						 TypeNameToString(t));
+					ereport(ERROR,
+							(errcode(ERRCODE_INVALID_FUNCTION_DEFINITION),
+					   errmsg("SQL function cannot accept shell type %s",
+							  TypeNameToString(t))));
 				else
-					elog(NOTICE, "Argument type \"%s\" is only a shell",
-						 TypeNameToString(t));
+					ereport(NOTICE,
+							(errcode(ERRCODE_WRONG_OBJECT_TYPE),
+							 errmsg("argument type %s is only a shell",
+									TypeNameToString(t))));
 			}
 		}
 		else
 		{
-			elog(ERROR, "Type \"%s\" does not exist",
-				 TypeNameToString(t));
+			ereport(ERROR,
+					(errcode(ERRCODE_UNDEFINED_OBJECT),
+					 errmsg("type %s does not exist",
+							TypeNameToString(t))));
 		}
 
 		if (t->setof)
-			elog(ERROR, "Functions cannot accept set arguments");
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_FUNCTION_DEFINITION),
+					 errmsg("functions cannot accept set arguments")));
 
 		parameterTypes[parameterCount++] = toid;
 	}
@@ -197,46 +215,61 @@ compute_attributes_sql_style(const List *options,
 		if (strcmp(defel->defname, "as") == 0)
 		{
 			if (as_item)
-				elog(ERROR, "conflicting or redundant options");
+				ereport(ERROR,
+						(errcode(ERRCODE_SYNTAX_ERROR),
+						 errmsg("conflicting or redundant options")));
 			as_item = defel;
 		}
 		else if (strcmp(defel->defname, "language") == 0)
 		{
 			if (language_item)
-				elog(ERROR, "conflicting or redundant options");
+				ereport(ERROR,
+						(errcode(ERRCODE_SYNTAX_ERROR),
+						 errmsg("conflicting or redundant options")));
 			language_item = defel;
 		}
 		else if (strcmp(defel->defname, "volatility") == 0)
 		{
 			if (volatility_item)
-				elog(ERROR, "conflicting or redundant options");
+				ereport(ERROR,
+						(errcode(ERRCODE_SYNTAX_ERROR),
+						 errmsg("conflicting or redundant options")));
 			volatility_item = defel;
 		}
 		else if (strcmp(defel->defname, "strict") == 0)
 		{
 			if (strict_item)
-				elog(ERROR, "conflicting or redundant options");
+				ereport(ERROR,
+						(errcode(ERRCODE_SYNTAX_ERROR),
+						 errmsg("conflicting or redundant options")));
 			strict_item = defel;
 		}
 		else if (strcmp(defel->defname, "security") == 0)
 		{
 			if (security_item)
-				elog(ERROR, "conflicting or redundant options");
+				ereport(ERROR,
+						(errcode(ERRCODE_SYNTAX_ERROR),
+						 errmsg("conflicting or redundant options")));
 			security_item = defel;
 		}
 		else
-			elog(ERROR, "invalid CREATE FUNCTION option");
+			elog(ERROR, "option \"%s\" not recognized",
+				 defel->defname);
 	}
 
 	if (as_item)
 		*as = (List *) as_item->arg;
 	else
-		elog(ERROR, "no function body specified");
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_FUNCTION_DEFINITION),
+				 errmsg("no function body specified")));
 
 	if (language_item)
 		*language = strVal(language_item->arg);
 	else
-		elog(ERROR, "no language specified");
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_FUNCTION_DEFINITION),
+				 errmsg("no language specified")));
 
 	if (volatility_item)
 	{
@@ -247,7 +280,8 @@ compute_attributes_sql_style(const List *options,
 		else if (strcmp(strVal(volatility_item->arg), "volatile") == 0)
 			*volatility_p = PROVOLATILE_VOLATILE;
 		else
-			elog(ERROR, "invalid volatility");
+			elog(ERROR, "invalid volatility \"%s\"",
+				 strVal(volatility_item->arg));
 	}
 
 	if (strict_item)
@@ -294,8 +328,10 @@ compute_attributes_with_style(List *parameters, bool *isStrict_p, char *volatili
 			*volatility_p = PROVOLATILE_IMMUTABLE;
 		}
 		else
-			elog(WARNING, "Unrecognized function attribute '%s' ignored",
-				 param->defname);
+			ereport(WARNING,
+					(errcode(ERRCODE_SYNTAX_ERROR),
+				 errmsg("unrecognized function attribute \"%s\" ignored",
+						param->defname)));
 	}
 }
 
@@ -336,8 +372,10 @@ interpret_AS_clause(Oid languageOid, const char *languageName, const List *as,
 		*probin_str_p = "-";
 
 		if (lnext(as) != NIL)
-			elog(ERROR, "CREATE FUNCTION: only one AS item needed for %s language",
-				 languageName);
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_FUNCTION_DEFINITION),
+					 errmsg("only one AS item needed for language \"%s\"",
+							languageName)));
 	}
 }
 
@@ -355,7 +393,7 @@ CreateFunction(CreateFunctionStmt *stmt)
 	Oid			prorettype;
 	bool		returnsSet;
 	char	   *language;
-	char		languageName[NAMEDATALEN];
+	char	   *languageName;
 	Oid			languageOid;
 	Oid			languageValidator;
 	char	   *funcname;
@@ -377,7 +415,8 @@ CreateFunction(CreateFunctionStmt *stmt)
 	/* Check we have creation rights in target namespace */
 	aclresult = pg_namespace_aclcheck(namespaceId, GetUserId(), ACL_CREATE);
 	if (aclresult != ACLCHECK_OK)
-		aclcheck_error(aclresult, get_namespace_name(namespaceId));
+		aclcheck_error(aclresult, ACL_KIND_NAMESPACE,
+					   get_namespace_name(namespaceId));
 
 	/* defaults attributes */
 	isStrict = false;
@@ -389,15 +428,25 @@ CreateFunction(CreateFunctionStmt *stmt)
 			   &as_clause, &language, &volatility, &isStrict, &security);
 
 	/* Convert language name to canonical case */
-	case_translate_language_name(language, languageName);
+	languageName = case_translate_language_name(language);
 
 	/* Look up the language and validate permissions */
 	languageTuple = SearchSysCache(LANGNAME,
 								   PointerGetDatum(languageName),
 								   0, 0, 0);
 	if (!HeapTupleIsValid(languageTuple))
-		elog(ERROR, "language \"%s\" does not exist", languageName);
-
+		/* Add any new languages to this list to invoke the hint. */
+		ereport(ERROR,
+				(errcode(ERRCODE_UNDEFINED_OBJECT),
+				 errmsg("language \"%s\" does not exist", languageName),
+				   (strcmp(languageName, "plperl") == 0 ||
+					strcmp(languageName, "plperlu") == 0 ||
+					strcmp(languageName, "plpgsql") == 0 ||
+					strcmp(languageName, "plpythonu") == 0 ||
+					strcmp(languageName, "pltcl") == 0 ||
+					strcmp(languageName, "pltclu") == 0) ?
+				 errhint("You need to use \"createlang\" to load the language into the database.") : 0));
+	
 	languageOid = HeapTupleGetOid(languageTuple);
 	languageStruct = (Form_pg_language) GETSTRUCT(languageTuple);
 
@@ -408,13 +457,15 @@ CreateFunction(CreateFunctionStmt *stmt)
 
 		aclresult = pg_language_aclcheck(languageOid, GetUserId(), ACL_USAGE);
 		if (aclresult != ACLCHECK_OK)
-			aclcheck_error(aclresult, NameStr(languageStruct->lanname));
+			aclcheck_error(aclresult, ACL_KIND_LANGUAGE,
+						   NameStr(languageStruct->lanname));
 	}
 	else
 	{
 		/* if untrusted language, must be superuser */
 		if (!superuser())
-			aclcheck_error(ACLCHECK_NO_PRIV, NameStr(languageStruct->lanname));
+			aclcheck_error(ACLCHECK_NO_PRIV, ACL_KIND_LANGUAGE,
+						   NameStr(languageStruct->lanname));
 	}
 
 	languageValidator = languageStruct->lanvalidator;
@@ -495,32 +546,35 @@ RemoveFunction(RemoveFuncStmt *stmt)
 	/*
 	 * Find the function, do permissions and validity checks
 	 */
-	funcOid = LookupFuncNameTypeNames(functionName, argTypes,
-									  "RemoveFunction");
+	funcOid = LookupFuncNameTypeNames(functionName, argTypes, false);
 
 	tup = SearchSysCache(PROCOID,
 						 ObjectIdGetDatum(funcOid),
 						 0, 0, 0);
 	if (!HeapTupleIsValid(tup)) /* should not happen */
-		elog(ERROR, "RemoveFunction: couldn't find tuple for function %s",
-			 NameListToString(functionName));
+		elog(ERROR, "cache lookup failed for function %u", funcOid);
 
 	/* Permission check: must own func or its namespace */
 	if (!pg_proc_ownercheck(funcOid, GetUserId()) &&
 		!pg_namespace_ownercheck(((Form_pg_proc) GETSTRUCT(tup))->pronamespace,
 								 GetUserId()))
-		aclcheck_error(ACLCHECK_NOT_OWNER, NameListToString(functionName));
+		aclcheck_error(ACLCHECK_NOT_OWNER, ACL_KIND_PROC,
+					   NameListToString(functionName));
 
 	if (((Form_pg_proc) GETSTRUCT(tup))->proisagg)
-		elog(ERROR, "RemoveFunction: function '%s' is an aggregate"
-			 "\n\tUse DROP AGGREGATE to remove it",
-			 NameListToString(functionName));
+		ereport(ERROR,
+				(errcode(ERRCODE_WRONG_OBJECT_TYPE),
+				 errmsg("\"%s\" is an aggregate function",
+						NameListToString(functionName)),
+			errhint("Use DROP AGGREGATE to drop aggregate functions.")));
 
 	if (((Form_pg_proc) GETSTRUCT(tup))->prolang == INTERNALlanguageId)
 	{
 		/* "Helpful" NOTICE when removing a builtin function ... */
-		elog(NOTICE, "Removing built-in function \"%s\"",
-			 NameListToString(functionName));
+		ereport(NOTICE,
+				(errcode(ERRCODE_WARNING),
+				 errmsg("removing built-in function \"%s\"",
+						NameListToString(functionName))));
 	}
 
 	ReleaseSysCache(tup);
@@ -557,8 +611,7 @@ RemoveFunctionById(Oid funcOid)
 						 ObjectIdGetDatum(funcOid),
 						 0, 0, 0);
 	if (!HeapTupleIsValid(tup)) /* should not happen */
-		elog(ERROR, "RemoveFunctionById: couldn't find tuple for function %u",
-			 funcOid);
+		elog(ERROR, "cache lookup failed for function %u", funcOid);
 
 	isagg = ((Form_pg_proc) GETSTRUCT(tup))->proisagg;
 
@@ -579,8 +632,7 @@ RemoveFunctionById(Oid funcOid)
 							 ObjectIdGetDatum(funcOid),
 							 0, 0, 0);
 		if (!HeapTupleIsValid(tup))		/* should not happen */
-			elog(ERROR, "RemoveFunctionById: couldn't find pg_aggregate tuple for %u",
-				 funcOid);
+			elog(ERROR, "cache lookup failed for pg_aggregate tuple for function %u", funcOid);
 
 		simple_heap_delete(relation, &tup->t_self);
 
@@ -591,12 +643,82 @@ RemoveFunctionById(Oid funcOid)
 }
 
 
+/*
+ * Rename function
+ */
+void
+RenameFunction(List *name, List *argtypes, const char *newname)
+{
+	Oid			procOid;
+	Oid			namespaceOid;
+	HeapTuple	tup;
+	Form_pg_proc procForm;
+	Relation	rel;
+	AclResult	aclresult;
+
+	rel = heap_openr(ProcedureRelationName, RowExclusiveLock);
+
+	procOid = LookupFuncNameTypeNames(name, argtypes, false);
+
+	tup = SearchSysCacheCopy(PROCOID,
+							 ObjectIdGetDatum(procOid),
+							 0, 0, 0);
+	if (!HeapTupleIsValid(tup)) /* should not happen */
+		elog(ERROR, "cache lookup failed for function %u", procOid);
+	procForm = (Form_pg_proc) GETSTRUCT(tup);
+
+	if (procForm->proisagg)
+		ereport(ERROR,
+				(errcode(ERRCODE_WRONG_OBJECT_TYPE),
+				 errmsg("\"%s\" is an aggregate function",
+						NameListToString(name)),
+		 errhint("Use ALTER AGGREGATE to rename aggregate functions.")));
+
+	namespaceOid = procForm->pronamespace;
+
+	/* make sure the new name doesn't exist */
+	if (SearchSysCacheExists(PROCNAMENSP,
+							 CStringGetDatum(newname),
+							 Int16GetDatum(procForm->pronargs),
+							 PointerGetDatum(procForm->proargtypes),
+							 ObjectIdGetDatum(namespaceOid)))
+	{
+		ereport(ERROR,
+				(errcode(ERRCODE_DUPLICATE_FUNCTION),
+				 errmsg("function %s already exists in schema \"%s\"",
+						funcname_signature_string(newname,
+												  procForm->pronargs,
+												  procForm->proargtypes),
+						get_namespace_name(namespaceOid))));
+	}
+
+	/* must be owner */
+	if (!pg_proc_ownercheck(procOid, GetUserId()))
+		aclcheck_error(ACLCHECK_NOT_OWNER, ACL_KIND_PROC,
+					   NameListToString(name));
+
+	/* must have CREATE privilege on namespace */
+	aclresult = pg_namespace_aclcheck(namespaceOid, GetUserId(), ACL_CREATE);
+	if (aclresult != ACLCHECK_OK)
+		aclcheck_error(aclresult, ACL_KIND_NAMESPACE,
+					   get_namespace_name(namespaceOid));
+
+	/* rename */
+	namestrcpy(&(procForm->proname), newname);
+	simple_heap_update(rel, &tup->t_self, tup);
+	CatalogUpdateIndexes(rel, tup);
+
+	heap_close(rel, NoLock);
+	heap_freetuple(tup);
+}
+
+
 /*
  * SetFunctionReturnType - change declared return type of a function
  *
  * This is presently only used for adjusting legacy functions that return
  * OPAQUE to return whatever we find their correct definition should be.
- * The caller should emit a suitable NOTICE explaining what we did.
+ * The caller should emit a suitable warning explaining what we did.
  */
 void
 SetFunctionReturnType(Oid funcOid, Oid newRetType)
@@ -611,13 +733,11 @@ SetFunctionReturnType(Oid funcOid, Oid newRetType)
 							 ObjectIdGetDatum(funcOid),
 							 0, 0, 0);
 	if (!HeapTupleIsValid(tup)) /* should not happen */
-		elog(ERROR, "SetFunctionReturnType: couldn't find tuple for function %u",
-			 funcOid);
+		elog(ERROR, "cache lookup failed for function %u", funcOid);
 	procForm = (Form_pg_proc) GETSTRUCT(tup);
 
-	if (procForm->prorettype != OPAQUEOID)
-		elog(ERROR, "SetFunctionReturnType: function %u doesn't return OPAQUE",
-			 funcOid);
+	if (procForm->prorettype != OPAQUEOID)		/* caller messed up */
+		elog(ERROR, "function %u doesn't return OPAQUE", funcOid);
 
 	/* okay to overwrite copied tuple */
 	procForm->prorettype = newRetType;
@@ -649,14 +769,12 @@ SetFunctionArgType(Oid funcOid, int argIndex, Oid newArgType)
 							 ObjectIdGetDatum(funcOid),
 							 0, 0, 0);
 	if (!HeapTupleIsValid(tup)) /* should not happen */
-		elog(ERROR, "SetFunctionArgType: couldn't find tuple for function %u",
-			 funcOid);
+		elog(ERROR, "cache lookup failed for function %u", funcOid);
 	procForm = (Form_pg_proc) GETSTRUCT(tup);
 
 	if (argIndex < 0 || argIndex >= procForm->pronargs ||
 		procForm->proargtypes[argIndex] != OPAQUEOID)
-		elog(ERROR, "SetFunctionArgType: function %u doesn't take OPAQUE",
-			 funcOid);
+		elog(ERROR, "function %u doesn't take OPAQUE", funcOid);
 
 	/* okay to overwrite copied tuple */
 	procForm->proargtypes[argIndex] = newArgType;
@@ -690,39 +808,56 @@ CreateCast(CreateCastStmt *stmt)
 
 	sourcetypeid = LookupTypeName(stmt->sourcetype);
 	if (!OidIsValid(sourcetypeid))
-		elog(ERROR, "source data type %s does not exist",
-			 TypeNameToString(stmt->sourcetype));
+		ereport(ERROR,
+				(errcode(ERRCODE_UNDEFINED_OBJECT),
+				 errmsg("source data type %s does not exist",
+						TypeNameToString(stmt->sourcetype))));
 
 	targettypeid = LookupTypeName(stmt->targettype);
 	if (!OidIsValid(targettypeid))
-		elog(ERROR, "target data type %s does not exist",
-			 TypeNameToString(stmt->targettype));
+		ereport(ERROR,
+				(errcode(ERRCODE_UNDEFINED_OBJECT),
+				 errmsg("target data type %s does not exist",
+						TypeNameToString(stmt->targettype))));
 
 	if (sourcetypeid == targettypeid)
-		elog(ERROR, "source data type and target data type are the same");
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_OBJECT_DEFINITION),
+		  errmsg("source data type and target data type are the same")));
 
 	/* No shells, no pseudo-types allowed */
 	if (!get_typisdefined(sourcetypeid))
-		elog(ERROR, "source data type %s is only a shell",
-			 TypeNameToString(stmt->sourcetype));
+		ereport(ERROR,
+				(errcode(ERRCODE_WRONG_OBJECT_TYPE),
+				 errmsg("source data type %s is only a shell",
+						TypeNameToString(stmt->sourcetype))));
 
 	if (!get_typisdefined(targettypeid))
-		elog(ERROR, "target data type %s is only a shell",
-			 TypeNameToString(stmt->targettype));
+		ereport(ERROR,
+				(errcode(ERRCODE_WRONG_OBJECT_TYPE),
+				 errmsg("target data type %s is only a shell",
+						TypeNameToString(stmt->targettype))));
 
 	if (get_typtype(sourcetypeid) == 'p')
-		elog(ERROR, "source data type %s is a pseudo-type",
-			 TypeNameToString(stmt->sourcetype));
+		ereport(ERROR,
+				(errcode(ERRCODE_WRONG_OBJECT_TYPE),
+				 errmsg("source data type %s is a pseudo-type",
+						TypeNameToString(stmt->sourcetype))));
 
 	if (get_typtype(targettypeid) == 'p')
-		elog(ERROR, "target data type %s is a pseudo-type",
-			 TypeNameToString(stmt->targettype));
+		ereport(ERROR,
+				(errcode(ERRCODE_WRONG_OBJECT_TYPE),
+				 errmsg("target data type %s is a pseudo-type",
+						TypeNameToString(stmt->targettype))));
 
+	/* Permission check */
 	if (!pg_type_ownercheck(sourcetypeid, GetUserId())
 		&& !pg_type_ownercheck(targettypeid, GetUserId()))
-		elog(ERROR, "must be owner of type %s or type %s",
-			 TypeNameToString(stmt->sourcetype),
-			 TypeNameToString(stmt->targettype));
+		ereport(ERROR,
+				(errcode(ERRCODE_INSUFFICIENT_PRIVILEGE),
+				 errmsg("must be owner of type %s or type %s",
+						TypeNameToString(stmt->sourcetype),
+						TypeNameToString(stmt->targettype))));
 
 	if (stmt->func != NULL)
 	{
@@ -730,45 +865,58 @@ CreateCast(CreateCastStmt *stmt)
 
 		funcid = LookupFuncNameTypeNames(stmt->func->funcname,
 										 stmt->func->funcargs,
-										 "CreateCast");
+										 false);
 
 		tuple = SearchSysCache(PROCOID,
 							   ObjectIdGetDatum(funcid),
 							   0, 0, 0);
 		if (!HeapTupleIsValid(tuple))
-			elog(ERROR, "cache lookup of function %u failed", funcid);
+			elog(ERROR, "cache lookup failed for function %u", funcid);
 
 		procstruct = (Form_pg_proc) GETSTRUCT(tuple);
 		if (procstruct->pronargs != 1)
-			elog(ERROR, "cast function must take 1 argument");
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_OBJECT_DEFINITION),
+					 errmsg("cast function must take one argument")));
 		if (procstruct->proargtypes[0] != sourcetypeid)
-			elog(ERROR, "argument of cast function must match source data type");
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_OBJECT_DEFINITION),
+					 errmsg("argument of cast function must match source data type")));
 		if (procstruct->prorettype != targettypeid)
-			elog(ERROR, "return data type of cast function must match target data type");
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_OBJECT_DEFINITION),
+					 errmsg("return data type of cast function must match target data type")));
+
 		/*
 		 * Restricting the volatility of a cast function may or may not be
 		 * a good idea in the abstract, but it definitely breaks many old
-		 * user-defined types.  Disable this check --- tgl 2/1/03
+		 * user-defined types.	Disable this check --- tgl 2/1/03
 		 */
 #ifdef NOT_USED
 		if (procstruct->provolatile == PROVOLATILE_VOLATILE)
-			elog(ERROR, "cast function must not be volatile");
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_OBJECT_DEFINITION),
+					 errmsg("cast function must not be volatile")));
 #endif
 		if (procstruct->proisagg)
-			elog(ERROR, "cast function must not be an aggregate function");
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_OBJECT_DEFINITION),
+			 errmsg("cast function must not be an aggregate function")));
 		if (procstruct->proretset)
-			elog(ERROR, "cast function must not return a set");
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_OBJECT_DEFINITION),
+					 errmsg("cast function must not return a set")));
 
 		ReleaseSysCache(tuple);
 	}
 	else
 	{
-		int16	typ1len;
-		int16	typ2len;
-		bool	typ1byval;
-		bool	typ2byval;
-		char	typ1align;
-		char	typ2align;
+		int16		typ1len;
+		int16		typ2len;
+		bool		typ1byval;
+		bool		typ2byval;
+		char		typ1align;
+		char		typ2align;
 
 		/* indicates binary coercibility */
 		funcid = InvalidOid;
@@ -778,12 +926,14 @@ CreateCast(CreateCastStmt *stmt)
 		 * erroneous casts can easily crash the backend.
 		 */
 		if (!superuser())
-			elog(ERROR, "Must be superuser to create a cast WITHOUT FUNCTION");
+			ereport(ERROR,
+					(errcode(ERRCODE_INSUFFICIENT_PRIVILEGE),
+					 errmsg("must be superuser to create a cast WITHOUT FUNCTION")));
 
 		/*
 		 * Also, insist that the types match as to size, alignment, and
 		 * pass-by-value attributes; this provides at least a crude check
-		 * that they have similar representations.  A pair of types that
+		 * that they have similar representations.	A pair of types that
 		 * fail this test should certainly not be equated.
 		 */
 		get_typlenbyvalalign(sourcetypeid, &typ1len, &typ1byval, &typ1align);
@@ -791,7 +941,9 @@ CreateCast(CreateCastStmt *stmt)
 		if (typ1len != typ2len ||
 			typ1byval != typ2byval ||
 			typ1align != typ2align)
-			elog(ERROR, "source and target datatypes are not physically compatible");
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_OBJECT_DEFINITION),
+					 errmsg("source and target data types are not physically compatible")));
 	}
 
 	/* convert CoercionContext enum to char value for castcontext */
@@ -807,7 +959,7 @@ CreateCast(CreateCastStmt *stmt)
 			castcontext = COERCION_CODE_EXPLICIT;
 			break;
 		default:
-			elog(ERROR, "CreateCast: bogus CoercionContext %c", stmt->context);
+			elog(ERROR, "unrecognized CoercionContext: %d", stmt->context);
 			castcontext = 0;	/* keep compiler quiet */
 			break;
 	}
@@ -815,18 +967,20 @@ CreateCast(CreateCastStmt *stmt)
 	relation = heap_openr(CastRelationName, RowExclusiveLock);
 
 	/*
-	 * Check for duplicate.  This is just to give a friendly error message,
-	 * the unique index would catch it anyway (so no need to sweat about
-	 * race conditions).
+	 * Check for duplicate.  This is just to give a friendly error
+	 * message, the unique index would catch it anyway (so no need to
+	 * sweat about race conditions).
 	 */
 	tuple = SearchSysCache(CASTSOURCETARGET,
 						   ObjectIdGetDatum(sourcetypeid),
 						   ObjectIdGetDatum(targettypeid),
 						   0, 0);
 	if (HeapTupleIsValid(tuple))
-		elog(ERROR, "cast from data type %s to data type %s already exists",
-			 TypeNameToString(stmt->sourcetype),
-			 TypeNameToString(stmt->targettype));
+		ereport(ERROR,
+				(errcode(ERRCODE_DUPLICATE_OBJECT),
+				 errmsg("cast from type %s to type %s already exists",
+						TypeNameToString(stmt->sourcetype),
+						TypeNameToString(stmt->targettype))));
 
 	/* ready to go */
 	values[Anum_pg_cast_castsource - 1] = ObjectIdGetDatum(sourcetypeid);
@@ -888,29 +1042,37 @@ DropCast(DropCastStmt *stmt)
 
 	sourcetypeid = LookupTypeName(stmt->sourcetype);
 	if (!OidIsValid(sourcetypeid))
-		elog(ERROR, "source data type %s does not exist",
-			 TypeNameToString(stmt->sourcetype));
+		ereport(ERROR,
+				(errcode(ERRCODE_UNDEFINED_OBJECT),
+				 errmsg("source data type %s does not exist",
+						TypeNameToString(stmt->sourcetype))));
 
 	targettypeid = LookupTypeName(stmt->targettype);
 	if (!OidIsValid(targettypeid))
-		elog(ERROR, "target data type %s does not exist",
-			 TypeNameToString(stmt->targettype));
+		ereport(ERROR,
+				(errcode(ERRCODE_UNDEFINED_OBJECT),
+				 errmsg("target data type %s does not exist",
+						TypeNameToString(stmt->targettype))));
 
 	tuple = SearchSysCache(CASTSOURCETARGET,
 						   ObjectIdGetDatum(sourcetypeid),
 						   ObjectIdGetDatum(targettypeid),
 						   0, 0);
 	if (!HeapTupleIsValid(tuple))
-		elog(ERROR, "cast from type %s to type %s does not exist",
-			 TypeNameToString(stmt->sourcetype),
-			 TypeNameToString(stmt->targettype));
+		ereport(ERROR,
+				(errcode(ERRCODE_UNDEFINED_OBJECT),
+				 errmsg("cast from type %s to type %s does not exist",
+						TypeNameToString(stmt->sourcetype),
+						TypeNameToString(stmt->targettype))));
 
 	/* Permission check */
 	if (!pg_type_ownercheck(sourcetypeid, GetUserId())
 		&& !pg_type_ownercheck(targettypeid, GetUserId()))
-		elog(ERROR, "must be owner of type %s or type %s",
-			 TypeNameToString(stmt->sourcetype),
-			 TypeNameToString(stmt->targettype));
+		ereport(ERROR,
+				(errcode(ERRCODE_INSUFFICIENT_PRIVILEGE),
+				 errmsg("must be owner of type %s or type %s",
+						TypeNameToString(stmt->sourcetype),
+						TypeNameToString(stmt->targettype))));
 
 	/*
 	 * Do the deletion
@@ -928,25 +1090,25 @@ DropCast(DropCastStmt *stmt)
 void
 DropCastById(Oid castOid)
 {
-	Relation	relation,
-				index;
+	Relation	relation;
 	ScanKeyData scankey;
-	IndexScanDesc scan;
+	SysScanDesc scan;
 	HeapTuple	tuple;
 
 	relation = heap_openr(CastRelationName, RowExclusiveLock);
-	index = index_openr(CastOidIndex);
 
 	ScanKeyEntryInitialize(&scankey, 0x0,
-						   1, F_OIDEQ, ObjectIdGetDatum(castOid));
-	scan = index_beginscan(relation, index, SnapshotNow, 1, &scankey);
-	tuple = index_getnext(scan, ForwardScanDirection);
-	if (HeapTupleIsValid(tuple))
-		simple_heap_delete(relation, &tuple->t_self);
-	else
-		elog(ERROR, "could not find tuple for cast %u", castOid);
-	index_endscan(scan);
+						   ObjectIdAttributeNumber,
+						   F_OIDEQ,
+						   ObjectIdGetDatum(castOid));
+	scan = systable_beginscan(relation, CastOidIndex, true,
+							  SnapshotNow, 1, &scankey);
 
-	index_close(index);
+	tuple = systable_getnext(scan);
+	if (!HeapTupleIsValid(tuple))
+		elog(ERROR, "could not find tuple for cast %u", castOid);
+	simple_heap_delete(relation, &tuple->t_self);
+
+	systable_endscan(scan);
 	heap_close(relation, RowExclusiveLock);
 }
