@@ -3,11 +3,11 @@
  * proclang.c
  *	  PostgreSQL PROCEDURAL LANGUAGE support code.
  *
- * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/commands/proclang.c,v 1.43 2002/09/21 18:39:25 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/commands/proclang.c,v 1.51.2.1 2004/02/21 00:35:13 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -40,11 +40,12 @@
 void
 CreateProceduralLanguage(CreatePLangStmt *stmt)
 {
-	char		languageName[NAMEDATALEN];
+	char	   *languageName;
 	Oid			procOid,
 				valProcOid;
 	Oid			funcrettype;
 	Oid			typev[FUNC_MAX_ARGS];
+	NameData	langname;
 	char		nulls[Natts_pg_language];
 	Datum		values[Natts_pg_language];
 	Relation	rel;
@@ -58,56 +59,57 @@ CreateProceduralLanguage(CreatePLangStmt *stmt)
 	 * Check permission
 	 */
 	if (!superuser())
-		elog(ERROR, "Only users with superuser privilege are "
-			 "permitted to create procedural languages");
+		ereport(ERROR,
+				(errcode(ERRCODE_INSUFFICIENT_PRIVILEGE),
+			 errmsg("must be superuser to create procedural language")));
 
 	/*
 	 * Translate the language name and check that this language doesn't
 	 * already exist
 	 */
-	case_translate_language_name(stmt->plname, languageName);
+	languageName = case_translate_language_name(stmt->plname);
 
 	if (SearchSysCacheExists(LANGNAME,
 							 PointerGetDatum(languageName),
 							 0, 0, 0))
-		elog(ERROR, "Language %s already exists", languageName);
+		ereport(ERROR,
+				(errcode(ERRCODE_DUPLICATE_OBJECT),
+				 errmsg("language \"%s\" already exists", languageName)));
 
 	/*
 	 * Lookup the PL handler function and check that it is of the expected
 	 * return type
 	 */
 	MemSet(typev, 0, sizeof(typev));
-	procOid = LookupFuncName(stmt->plhandler, 0, typev);
-	if (!OidIsValid(procOid))
-		elog(ERROR, "function %s() doesn't exist",
-			 NameListToString(stmt->plhandler));
+	procOid = LookupFuncName(stmt->plhandler, 0, typev, false);
 	funcrettype = get_func_rettype(procOid);
 	if (funcrettype != LANGUAGE_HANDLEROID)
 	{
 		/*
-		 * We allow OPAQUE just so we can load old dump files.  When we
+		 * We allow OPAQUE just so we can load old dump files.	When we
 		 * see a handler function declared OPAQUE, change it to
 		 * LANGUAGE_HANDLER.
 		 */
 		if (funcrettype == OPAQUEOID)
 		{
-			elog(NOTICE, "CreateProceduralLanguage: changing return type of function %s() from OPAQUE to LANGUAGE_HANDLER",
-				 NameListToString(stmt->plhandler));
+			ereport(WARNING,
+					(errcode(ERRCODE_WRONG_OBJECT_TYPE),
+					 errmsg("changing return type of function %s from \"opaque\" to \"language_handler\"",
+							NameListToString(stmt->plhandler))));
 			SetFunctionReturnType(procOid, LANGUAGE_HANDLEROID);
 		}
 		else
-			elog(ERROR, "CreateProceduralLanguage: function %s() must return LANGUAGE_HANDLER",
-				 NameListToString(stmt->plhandler));
+			ereport(ERROR,
+					(errcode(ERRCODE_WRONG_OBJECT_TYPE),
+					 errmsg("function %s must return type \"language_handler\"",
+							NameListToString(stmt->plhandler))));
 	}
 
 	/* validate the validator function */
 	if (stmt->plvalidator)
 	{
 		typev[0] = OIDOID;
-		valProcOid = LookupFuncName(stmt->plvalidator, 1, typev);
-		if (!OidIsValid(valProcOid))
-			elog(ERROR, "function %s(oid) doesn't exist",
-				 NameListToString(stmt->plvalidator));
+		valProcOid = LookupFuncName(stmt->plvalidator, 1, typev, false);
 		/* return value is ignored, so we don't check the type */
 	}
 	else
@@ -123,12 +125,13 @@ CreateProceduralLanguage(CreatePLangStmt *stmt)
 	}
 
 	i = 0;
-	values[i++] = PointerGetDatum(languageName);
-	values[i++] = BoolGetDatum(true);	/* lanispl */
-	values[i++] = BoolGetDatum(stmt->pltrusted);
-	values[i++] = ObjectIdGetDatum(procOid);
-	values[i++] = ObjectIdGetDatum(valProcOid);
-	nulls[i] = 'n';				/* lanacl */
+	namestrcpy(&langname, languageName);
+	values[i++] = NameGetDatum(&langname);			/* lanname */
+	values[i++] = BoolGetDatum(true);				/* lanispl */
+	values[i++] = BoolGetDatum(stmt->pltrusted);	/* lanpltrusted */
+	values[i++] = ObjectIdGetDatum(procOid);		/* lanplcallfoid */
+	values[i++] = ObjectIdGetDatum(valProcOid);		/* lanvalidator */
+	nulls[i] = 'n';									/* lanacl */
 
 	rel = heap_openr(LanguageRelationName, RowExclusiveLock);
 
@@ -172,7 +175,7 @@ CreateProceduralLanguage(CreatePLangStmt *stmt)
 void
 DropProceduralLanguage(DropPLangStmt *stmt)
 {
-	char		languageName[NAMEDATALEN];
+	char	   *languageName;
 	HeapTuple	langTup;
 	ObjectAddress object;
 
@@ -180,20 +183,23 @@ DropProceduralLanguage(DropPLangStmt *stmt)
 	 * Check permission
 	 */
 	if (!superuser())
-		elog(ERROR, "Only users with superuser privilege are "
-			 "permitted to drop procedural languages");
+		ereport(ERROR,
+				(errcode(ERRCODE_INSUFFICIENT_PRIVILEGE),
+			   errmsg("must be superuser to drop procedural language")));
 
 	/*
 	 * Translate the language name, check that this language exist and is
 	 * a PL
 	 */
-	case_translate_language_name(stmt->plname, languageName);
+	languageName = case_translate_language_name(stmt->plname);
 
 	langTup = SearchSysCache(LANGNAME,
 							 CStringGetDatum(languageName),
 							 0, 0, 0);
 	if (!HeapTupleIsValid(langTup))
-		elog(ERROR, "Language %s doesn't exist", languageName);
+		ereport(ERROR,
+				(errcode(ERRCODE_UNDEFINED_OBJECT),
+				 errmsg("language \"%s\" does not exist", languageName)));
 
 	object.classId = get_system_catalog_relid(LanguageRelationName);
 	object.objectId = HeapTupleGetOid(langTup);
@@ -221,9 +227,8 @@ DropProceduralLanguageById(Oid langOid)
 	langTup = SearchSysCache(LANGOID,
 							 ObjectIdGetDatum(langOid),
 							 0, 0, 0);
-	if (!HeapTupleIsValid(langTup))
-		elog(ERROR, "DropProceduralLanguageById: language %u not found",
-			 langOid);
+	if (!HeapTupleIsValid(langTup))		/* should not happen */
+		elog(ERROR, "cache lookup failed for language %u", langOid);
 
 	simple_heap_delete(rel, &langTup->t_self);
 
@@ -231,3 +236,45 @@ DropProceduralLanguageById(Oid langOid)
 
 	heap_close(rel, RowExclusiveLock);
 }
+
+/*
+ * Rename language
+ */
+void
+RenameLanguage(const char *oldname, const char *newname)
+{
+	HeapTuple	tup;
+	Relation	rel;
+
+	rel = heap_openr(ShadowRelationName, RowExclusiveLock);
+
+	tup = SearchSysCacheCopy(LANGNAME,
+							 CStringGetDatum(oldname),
+							 0, 0, 0);
+	if (!HeapTupleIsValid(tup))
+		ereport(ERROR,
+				(errcode(ERRCODE_UNDEFINED_OBJECT),
+				 errmsg("language \"%s\" does not exist", oldname)));
+
+	/* make sure the new name doesn't exist */
+	if (SearchSysCacheExists(LANGNAME,
+							 CStringGetDatum(newname),
+							 0, 0, 0))
+		ereport(ERROR,
+				(errcode(ERRCODE_DUPLICATE_OBJECT),
+				 errmsg("language \"%s\" already exists", newname)));
+
+	/* must be superuser */
+	if (!superuser())
+		ereport(ERROR,
+				(errcode(ERRCODE_INSUFFICIENT_PRIVILEGE),
+			 errmsg("must be superuser to rename procedural language")));
+
+	/* rename */
+	namestrcpy(&(((Form_pg_language) GETSTRUCT(tup))->lanname), newname);
+	simple_heap_update(rel, &tup->t_self, tup);
+	CatalogUpdateIndexes(rel, tup);
+
+	heap_close(rel, NoLock);
+	heap_freetuple(tup);
+}
