@@ -7,12 +7,12 @@
  * we need two sets of code.  Ought to look at trying to unify the cases.
  *
  *
- * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/executor/nodeSubqueryscan.c,v 1.13 2002/06/20 20:29:28 momjian Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/executor/nodeSubqueryscan.c,v 1.22 2003/10/01 21:30:52 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -35,7 +35,7 @@
 #include "parser/parsetree.h"
 #include "tcop/pquery.h"
 
-static TupleTableSlot *SubqueryNext(SubqueryScan *node);
+static TupleTableSlot *SubqueryNext(SubqueryScanState *node);
 
 /* ----------------------------------------------------------------
  *						Scan Support
@@ -48,18 +48,17 @@ static TupleTableSlot *SubqueryNext(SubqueryScan *node);
  * ----------------------------------------------------------------
  */
 static TupleTableSlot *
-SubqueryNext(SubqueryScan *node)
+SubqueryNext(SubqueryScanState *node)
 {
-	SubqueryScanState *subquerystate;
 	EState	   *estate;
 	ScanDirection direction;
 	TupleTableSlot *slot;
+	MemoryContext oldcontext;
 
 	/*
 	 * get information from the estate and scan state
 	 */
-	estate = node->scan.plan.state;
-	subquerystate = (SubqueryScanState *) node->scan.scanstate;
+	estate = node->ss.ps.state;
 	direction = estate->es_direction;
 
 	/*
@@ -68,13 +67,18 @@ SubqueryNext(SubqueryScan *node)
 	 */
 
 	/*
-	 * get the next tuple from the sub-query
+	 * Get the next tuple from the sub-query.  We have to be careful to
+	 * run it in its appropriate memory context.
 	 */
-	subquerystate->sss_SubEState->es_direction = direction;
+	node->sss_SubEState->es_direction = direction;
 
-	slot = ExecProcNode(node->subplan, (Plan *) node);
+	oldcontext = MemoryContextSwitchTo(node->sss_SubEState->es_query_cxt);
 
-	subquerystate->csstate.css_ScanTupleSlot = slot;
+	slot = ExecProcNode(node->subplan);
+
+	MemoryContextSwitchTo(oldcontext);
+
+	node->ss.ss_ScanTupleSlot = slot;
 
 	return slot;
 }
@@ -90,55 +94,62 @@ SubqueryNext(SubqueryScan *node)
  */
 
 TupleTableSlot *
-ExecSubqueryScan(SubqueryScan *node)
+ExecSubqueryScan(SubqueryScanState *node)
 {
 	/*
 	 * use SubqueryNext as access method
 	 */
-	return ExecScan(&node->scan, (ExecScanAccessMtd) SubqueryNext);
+	return ExecScan(&node->ss, (ExecScanAccessMtd) SubqueryNext);
 }
 
 /* ----------------------------------------------------------------
  *		ExecInitSubqueryScan
  * ----------------------------------------------------------------
  */
-bool
-ExecInitSubqueryScan(SubqueryScan *node, EState *estate, Plan *parent)
+SubqueryScanState *
+ExecInitSubqueryScan(SubqueryScan *node, EState *estate)
 {
 	SubqueryScanState *subquerystate;
 	RangeTblEntry *rte;
 	EState	   *sp_estate;
+	MemoryContext oldcontext;
 
 	/*
 	 * SubqueryScan should not have any "normal" children.
 	 */
-	Assert(outerPlan((Plan *) node) == NULL);
-	Assert(innerPlan((Plan *) node) == NULL);
+	Assert(outerPlan(node) == NULL);
+	Assert(innerPlan(node) == NULL);
 
 	/*
-	 * assign the node's execution state
-	 */
-	node->scan.plan.state = estate;
-
-	/*
-	 * create new SubqueryScanState for node
+	 * create state structure
 	 */
 	subquerystate = makeNode(SubqueryScanState);
-	node->scan.scanstate = (CommonScanState *) subquerystate;
+	subquerystate->ss.ps.plan = (Plan *) node;
+	subquerystate->ss.ps.state = estate;
 
 	/*
 	 * Miscellaneous initialization
 	 *
 	 * create expression context for node
 	 */
-	ExecAssignExprContext(estate, &subquerystate->csstate.cstate);
+	ExecAssignExprContext(estate, &subquerystate->ss.ps);
+
+	/*
+	 * initialize child expressions
+	 */
+	subquerystate->ss.ps.targetlist = (List *)
+		ExecInitExpr((Expr *) node->scan.plan.targetlist,
+					 (PlanState *) subquerystate);
+	subquerystate->ss.ps.qual = (List *)
+		ExecInitExpr((Expr *) node->scan.plan.qual,
+					 (PlanState *) subquerystate);
 
 #define SUBQUERYSCAN_NSLOTS 1
 
 	/*
 	 * tuple table initialization
 	 */
-	ExecInitResultTupleSlot(estate, &subquerystate->csstate.cstate);
+	ExecInitResultTupleSlot(estate, &subquerystate->ss.ps);
 
 	/*
 	 * initialize subquery
@@ -148,29 +159,44 @@ ExecInitSubqueryScan(SubqueryScan *node, EState *estate, Plan *parent)
 	rte = rt_fetch(node->scan.scanrelid, estate->es_range_table);
 	Assert(rte->rtekind == RTE_SUBQUERY);
 
+	/*
+	 * The subquery needs its own EState because it has its own
+	 * rangetable. It shares our Param ID space, however.  XXX if
+	 * rangetable access were done differently, the subquery could share
+	 * our EState, which would eliminate some thrashing about in this
+	 * module...
+	 */
 	sp_estate = CreateExecutorState();
 	subquerystate->sss_SubEState = sp_estate;
 
+	oldcontext = MemoryContextSwitchTo(sp_estate->es_query_cxt);
+
 	sp_estate->es_range_table = rte->subquery->rtable;
 	sp_estate->es_param_list_info = estate->es_param_list_info;
 	sp_estate->es_param_exec_vals = estate->es_param_exec_vals;
 	sp_estate->es_tupleTable =
 		ExecCreateTupleTable(ExecCountSlotsNode(node->subplan) + 10);
 	sp_estate->es_snapshot = estate->es_snapshot;
-
-	if (!ExecInitNode(node->subplan, sp_estate, (Plan *) node))
-		return false;
-
-	subquerystate->csstate.css_ScanTupleSlot = NULL;
-	subquerystate->csstate.cstate.cs_TupFromTlist = false;
+	sp_estate->es_crosscheck_snapshot = estate->es_crosscheck_snapshot;
+	sp_estate->es_instrument = estate->es_instrument;
 
 	/*
-	 * initialize tuple type
+	 * Start up the subplan (this is a very cut-down form of InitPlan())
 	 */
-	ExecAssignResultTypeFromTL((Plan *) node, &subquerystate->csstate.cstate);
-	ExecAssignProjectionInfo((Plan *) node, &subquerystate->csstate.cstate);
+	subquerystate->subplan = ExecInitNode(node->subplan, sp_estate);
 
-	return TRUE;
+	MemoryContextSwitchTo(oldcontext);
+
+	subquerystate->ss.ss_ScanTupleSlot = NULL;
+	subquerystate->ss.ps.ps_TupFromTlist = false;
+
+	/*
+	 * Initialize result tuple type and projection info.
+	 */
+	ExecAssignResultTypeFromTL(&subquerystate->ss.ps);
+	ExecAssignProjectionInfo(&subquerystate->ss.ps);
+
+	return subquerystate;
 }
 
 int
@@ -191,42 +217,30 @@ ExecCountSlotsSubqueryScan(SubqueryScan *node)
  * ----------------------------------------------------------------
  */
 void
-ExecEndSubqueryScan(SubqueryScan *node)
+ExecEndSubqueryScan(SubqueryScanState *node)
 {
-	SubqueryScanState *subquerystate;
+	MemoryContext oldcontext;
 
 	/*
-	 * get information from node
+	 * Free the exprcontext
 	 */
-	subquerystate = (SubqueryScanState *) node->scan.scanstate;
-
-	/*
-	 * Free the projection info and the scan attribute info
-	 *
-	 * Note: we don't ExecFreeResultType(subquerystate) because the rule
-	 * manager depends on the tupType returned by ExecMain().  So for now,
-	 * this is freed at end-transaction time.  -cim 6/2/91
-	 */
-	ExecFreeProjectionInfo(&subquerystate->csstate.cstate);
-	ExecFreeExprContext(&subquerystate->csstate.cstate);
-
-	/*
-	 * close down subquery
-	 */
-	ExecEndNode(node->subplan, (Plan *) node);
-
-	/*
-	 * clean up subquery's tuple table
-	 */
-	subquerystate->csstate.css_ScanTupleSlot = NULL;
-	ExecDropTupleTable(subquerystate->sss_SubEState->es_tupleTable, true);
-
-	/* XXX we seem to be leaking the sub-EState... */
+	ExecFreeExprContext(&node->ss.ps);
 
 	/*
 	 * clean out the upper tuple table
 	 */
-	ExecClearTuple(subquerystate->csstate.cstate.cs_ResultTupleSlot);
+	ExecClearTuple(node->ss.ps.ps_ResultTupleSlot);
+
+	/*
+	 * close down subquery
+	 */
+	oldcontext = MemoryContextSwitchTo(node->sss_SubEState->es_query_cxt);
+
+	ExecEndPlan(node->subplan, node->sss_SubEState);
+
+	MemoryContextSwitchTo(oldcontext);
+
+	FreeExecutorState(node->sss_SubEState);
 }
 
 /* ----------------------------------------------------------------
@@ -236,27 +250,32 @@ ExecEndSubqueryScan(SubqueryScan *node)
  * ----------------------------------------------------------------
  */
 void
-ExecSubqueryReScan(SubqueryScan *node, ExprContext *exprCtxt, Plan *parent)
+ExecSubqueryReScan(SubqueryScanState *node, ExprContext *exprCtxt)
 {
-	SubqueryScanState *subquerystate;
 	EState	   *estate;
+	MemoryContext oldcontext;
 
-	subquerystate = (SubqueryScanState *) node->scan.scanstate;
-	estate = node->scan.plan.state;
+	estate = node->ss.ps.state;
+
+	oldcontext = MemoryContextSwitchTo(node->sss_SubEState->es_query_cxt);
 
 	/*
 	 * ExecReScan doesn't know about my subplan, so I have to do
-	 * changed-parameter signaling myself.
+	 * changed-parameter signaling myself.	This is just as well, because
+	 * the subplan has its own memory context in which its chgParam state
+	 * lives.
 	 */
-	if (node->scan.plan.chgParam != NULL)
-		SetChangedParamList(node->subplan, node->scan.plan.chgParam);
+	if (node->ss.ps.chgParam != NULL)
+		UpdateChangedParamSet(node->subplan, node->ss.ps.chgParam);
 
 	/*
 	 * if chgParam of subnode is not null then plan will be re-scanned by
 	 * first ExecProcNode.
 	 */
 	if (node->subplan->chgParam == NULL)
-		ExecReScan(node->subplan, NULL, (Plan *) node);
+		ExecReScan(node->subplan, NULL);
 
-	subquerystate->csstate.css_ScanTupleSlot = NULL;
+	MemoryContextSwitchTo(oldcontext);
+
+	node->ss.ss_ScanTupleSlot = NULL;
 }
