@@ -3,12 +3,12 @@
  * print.c
  *	  various print routines (used mostly for debugging)
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/nodes/print.c,v 1.51 2001/12/20 02:39:26 momjian Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/nodes/print.c,v 1.64 2003/08/11 23:04:49 tgl Exp $
  *
  * HISTORY
  *	  AUTHOR			DATE			MAJOR EVENT
@@ -21,6 +21,7 @@
 
 #include "access/printtup.h"
 #include "catalog/pg_type.h"
+#include "lib/stringinfo.h"
 #include "nodes/print.h"
 #include "optimizer/clauses.h"
 #include "parser/parsetree.h"
@@ -37,31 +38,131 @@ void
 print(void *obj)
 {
 	char	   *s;
+	char	   *f;
 
 	s = nodeToString(obj);
-	printf("%s\n", s);
-	fflush(stdout);
+	f = format_node_dump(s);
 	pfree(s);
+	printf("%s\n", f);
+	fflush(stdout);
+	pfree(f);
 }
 
 /*
- * pretty print hack extraordinaire.  -ay 10/94
+ * pprint
+ *	  pretty-print contents of Node to stdout
  */
 void
 pprint(void *obj)
 {
-#define INDENTSTOP	3
-#define MAXINDENT	60
-#define LINELEN		80
 	char	   *s;
-	int			i;
-	char		line[LINELEN];
-	int			indentLev;
-	int			indentDist;
-	int			j;
+	char	   *f;
 
 	s = nodeToString(obj);
+	f = pretty_format_node_dump(s);
+	pfree(s);
+	printf("%s\n", f);
+	fflush(stdout);
+	pfree(f);
+}
 
+/*
+ * elog_node_display
+ *	  send pretty-printed contents of Node to postmaster log
+ */
+void
+elog_node_display(int lev, const char *title, void *obj, bool pretty)
+{
+	char	   *s;
+	char	   *f;
+
+	s = nodeToString(obj);
+	if (pretty)
+		f = pretty_format_node_dump(s);
+	else
+		f = format_node_dump(s);
+	pfree(s);
+	ereport(lev,
+			(errmsg_internal("%s:", title),
+			 errdetail("%s", f)));
+	pfree(f);
+}
+
+/*
+ * Format a nodeToString output for display on a terminal.
+ *
+ * The result is a palloc'd string.
+ *
+ * This version just tries to break at whitespace.
+ */
+char *
+format_node_dump(const char *dump)
+{
+#define LINELEN		78
+	char		line[LINELEN + 1];
+	StringInfoData str;
+	int			i;
+	int			j;
+	int			k;
+
+	initStringInfo(&str);
+	i = 0;
+	for (;;)
+	{
+		for (j = 0; j < LINELEN && dump[i] != '\0'; i++, j++)
+			line[j] = dump[i];
+		if (dump[i] == '\0')
+			break;
+		if (dump[i] == ' ')
+		{
+			/* ok to break at adjacent space */
+			i++;
+		}
+		else
+		{
+			for (k = j - 1; k > 0; k--)
+				if (line[k] == ' ')
+					break;
+			if (k > 0)
+			{
+				/* back up; will reprint all after space */
+				i -= (j - k - 1);
+				j = k;
+			}
+		}
+		line[j] = '\0';
+		appendStringInfo(&str, "%s\n", line);
+	}
+	if (j > 0)
+	{
+		line[j] = '\0';
+		appendStringInfo(&str, "%s\n", line);
+	}
+	return str.data;
+#undef LINELEN
+}
+
+/*
+ * Format a nodeToString output for display on a terminal.
+ *
+ * The result is a palloc'd string.
+ *
+ * This version tries to indent intelligently.
+ */
+char *
+pretty_format_node_dump(const char *dump)
+{
+#define INDENTSTOP	3
+#define MAXINDENT	60
+#define LINELEN		78
+	char		line[LINELEN + 1];
+	StringInfoData str;
+	int			indentLev;
+	int			indentDist;
+	int			i;
+	int			j;
+
+	initStringInfo(&str);
 	indentLev = 0;				/* logical indent level */
 	indentDist = 0;				/* physical indent distance */
 	i = 0;
@@ -69,9 +170,9 @@ pprint(void *obj)
 	{
 		for (j = 0; j < indentDist; j++)
 			line[j] = ' ';
-		for (; j < LINELEN-1 && s[i] != '\0'; i++, j++)
+		for (; j < LINELEN && dump[i] != '\0'; i++, j++)
 		{
-			line[j] = s[i];
+			line[j] = dump[i];
 			switch (line[j])
 			{
 				case '}':
@@ -79,11 +180,12 @@ pprint(void *obj)
 					{
 						/* print data before the } */
 						line[j] = '\0';
-						printf("%s\n", line);
+						appendStringInfo(&str, "%s\n", line);
 					}
 					/* print the } at indentDist */
-					line[indentDist] = '\0';
-					printf("%s}\n", line);
+					line[indentDist] = '}';
+					line[indentDist + 1] = '\0';
+					appendStringInfo(&str, "%s\n", line);
 					/* outdent */
 					if (indentLev > 0)
 					{
@@ -96,7 +198,7 @@ pprint(void *obj)
 				case ')':
 					/* force line break after ')' */
 					line[j + 1] = '\0';
-					printf("%s\n", line);
+					appendStringInfo(&str, "%s\n", line);
 					j = indentDist - 1;
 					break;
 				case '{':
@@ -104,36 +206,38 @@ pprint(void *obj)
 					if (j != indentDist)
 					{
 						line[j] = '\0';
-						printf("%s\n", line);
+						appendStringInfo(&str, "%s\n", line);
 					}
 					/* indent */
 					indentLev++;
 					indentDist = Min(indentLev * INDENTSTOP, MAXINDENT);
 					for (j = 0; j < indentDist; j++)
 						line[j] = ' ';
-					line[j] = s[i];
+					line[j] = dump[i];
 					break;
 				case ':':
 					/* force line break before : */
 					if (j != indentDist)
 					{
 						line[j] = '\0';
-						printf("%s\n", line);
+						appendStringInfo(&str, "%s\n", line);
 					}
 					j = indentDist;
-					line[j] = s[i];
+					line[j] = dump[i];
 					break;
 			}
 		}
 		line[j] = '\0';
-		if (s[i] == '\0')
+		if (dump[i] == '\0')
 			break;
-		printf("%s\n", line);
+		appendStringInfo(&str, "%s\n", line);
 	}
-	if (j != 0)
-		printf("%s\n", line);
-	fflush(stdout);
-	pfree(s);
+	if (j > 0)
+		appendStringInfo(&str, "%s\n", line);
+	return str.data;
+#undef INDENTSTOP
+#undef MAXINDENT
+#undef LINELEN
 }
 
 /*
@@ -146,18 +250,39 @@ print_rt(List *rtable)
 	List	   *l;
 	int			i = 1;
 
-	printf("resno\trelname(refname)\trelid\tinFromCl\n");
-	printf("-----\t----------------\t-----\t--------\n");
+	printf("resno\trefname  \trelid\tinFromCl\n");
+	printf("-----\t---------\t-----\t--------\n");
 	foreach(l, rtable)
 	{
 		RangeTblEntry *rte = lfirst(l);
 
-		if (rte->relname)
-			printf("%d\t%s (%s)\t%u",
-				   i, rte->relname, rte->eref->relname, rte->relid);
-		else
-			printf("%d\t[subquery] (%s)\t",
-				   i, rte->eref->relname);
+		switch (rte->rtekind)
+		{
+			case RTE_RELATION:
+				printf("%d\t%s\t%u",
+					   i, rte->eref->aliasname, rte->relid);
+				break;
+			case RTE_SUBQUERY:
+				printf("%d\t%s\t[subquery]",
+					   i, rte->eref->aliasname);
+				break;
+			case RTE_FUNCTION:
+				printf("%d\t%s\t[rangefunction]",
+					   i, rte->eref->aliasname);
+				break;
+			case RTE_JOIN:
+				printf("%d\t%s\t[join]",
+					   i, rte->eref->aliasname);
+				break;
+			case RTE_SPECIAL:
+				printf("%d\t%s\t[special]",
+					   i, rte->eref->aliasname);
+				break;
+			default:
+				printf("%d\t%s\t[unknown rtekind]",
+					   i, rte->eref->aliasname);
+		}
+
 		printf("\t%s\t%s\n",
 			   (rte->inh ? "inh" : ""),
 			   (rte->inFromCl ? "inFromCl" : ""));
@@ -202,7 +327,7 @@ print_expr(Node *expr, List *rtable)
 					Assert(var->varno > 0 &&
 						   (int) var->varno <= length(rtable));
 					rte = rt_fetch(var->varno, rtable);
-					relname = rte->eref->relname;
+					relname = rte->eref->aliasname;
 					attname = get_rte_attribute_name(rte, var->varattno);
 				}
 				break;
@@ -227,7 +352,7 @@ print_expr(Node *expr, List *rtable)
 								 ObjectIdGetDatum(c->consttype),
 								 0, 0, 0);
 		if (!HeapTupleIsValid(typeTup))
-			elog(ERROR, "Cache lookup for type %u failed", c->consttype);
+			elog(ERROR, "cache lookup failed for type %u", c->consttype);
 		typoutput = ((Form_pg_type) GETSTRUCT(typeTup))->typoutput;
 		typelem = ((Form_pg_type) GETSTRUCT(typeTup))->typelem;
 		ReleaseSysCache(typeTup);
@@ -239,24 +364,43 @@ print_expr(Node *expr, List *rtable)
 		printf("%s", outputstr);
 		pfree(outputstr);
 	}
-	else if (IsA(expr, Expr))
+	else if (IsA(expr, OpExpr))
 	{
-		Expr	   *e = (Expr *) expr;
+		OpExpr	   *e = (OpExpr *) expr;
+		char	   *opname;
 
-		if (is_opclause(expr))
+		opname = get_opname(e->opno);
+		if (length(e->args) > 1)
 		{
-			char	   *opname;
-
-			print_expr((Node *) get_leftop(e), rtable);
-			opname = get_opname(((Oper *) e->oper)->opno);
+			print_expr(get_leftop((Expr *) e), rtable);
 			printf(" %s ", ((opname != NULL) ? opname : "(invalid operator)"));
-			print_expr((Node *) get_rightop(e), rtable);
+			print_expr(get_rightop((Expr *) e), rtable);
 		}
 		else
-			printf("an expr");
+		{
+			/* we print prefix and postfix ops the same... */
+			printf("%s ", ((opname != NULL) ? opname : "(invalid operator)"));
+			print_expr(get_leftop((Expr *) e), rtable);
+		}
+	}
+	else if (IsA(expr, FuncExpr))
+	{
+		FuncExpr   *e = (FuncExpr *) expr;
+		char	   *funcname;
+		List	   *l;
+
+		funcname = get_func_name(e->funcid);
+		printf("%s(", ((funcname != NULL) ? funcname : "(invalid function)"));
+		foreach(l, e->args)
+		{
+			print_expr(lfirst(l), rtable);
+			if (lnext(l))
+				printf(",");
+		}
+		printf(")");
 	}
 	else
-		printf("not an expr");
+		printf("unknown expr");
 }
 
 /*
@@ -304,12 +448,13 @@ print_tl(List *tlist, List *rtable)
 	{
 		TargetEntry *tle = lfirst(tl);
 
-		printf("\t%d %s\t", tle->resdom->resno, tle->resdom->resname);
-		if (tle->resdom->reskey != 0)
-			printf("(%d):\t", tle->resdom->reskey);
+		printf("\t%d %s\t", tle->resdom->resno,
+			   tle->resdom->resname ? tle->resdom->resname : "<null>");
+		if (tle->resdom->ressortgroupref != 0)
+			printf("(%u):\t", tle->resdom->ressortgroupref);
 		else
 			printf("    :\t");
-		print_expr(tle->expr, rtable);
+		print_expr((Node *) tle->expr, rtable);
 		printf("\n");
 	}
 	printf(")\n");
@@ -357,6 +502,8 @@ plannode_type(Plan *p)
 			return "TIDSCAN";
 		case T_SubqueryScan:
 			return "SUBQUERYSCAN";
+		case T_FunctionScan:
+			return "FUNCTIONSCAN";
 		case T_Join:
 			return "JOIN";
 		case T_NestLoop:
@@ -387,12 +534,8 @@ plannode_type(Plan *p)
 }
 
 /*
-   prints the ascii description of the plan nodes
-   does this recursively by doing a depth-first traversal of the
-   plan tree.  for SeqScan and IndexScan, the name of the table is also
-   printed out
-
-*/
+ * Recursively prints a simple text description of the plan tree
+ */
 void
 print_plan_recursive(Plan *p, Query *parsetree, int indentLevel, char *label)
 {
@@ -406,19 +549,27 @@ print_plan_recursive(Plan *p, Query *parsetree, int indentLevel, char *label)
 	printf("%s%s :c=%.2f..%.2f :r=%.0f :w=%d ", label, plannode_type(p),
 		   p->startup_cost, p->total_cost,
 		   p->plan_rows, p->plan_width);
-	if (IsA(p, Scan) ||IsA(p, SeqScan))
+	if (IsA(p, Scan) ||
+		IsA(p, SeqScan))
 	{
 		RangeTblEntry *rte;
 
 		rte = rt_fetch(((Scan *) p)->scanrelid, parsetree->rtable);
-		StrNCpy(extraInfo, rte->relname, NAMEDATALEN);
+		StrNCpy(extraInfo, rte->eref->aliasname, NAMEDATALEN);
 	}
 	else if (IsA(p, IndexScan))
 	{
 		RangeTblEntry *rte;
 
 		rte = rt_fetch(((IndexScan *) p)->scan.scanrelid, parsetree->rtable);
-		StrNCpy(extraInfo, rte->relname, NAMEDATALEN);
+		StrNCpy(extraInfo, rte->eref->aliasname, NAMEDATALEN);
+	}
+	else if (IsA(p, FunctionScan))
+	{
+		RangeTblEntry *rte;
+
+		rte = rt_fetch(((FunctionScan *) p)->scan.scanrelid, parsetree->rtable);
+		StrNCpy(extraInfo, rte->eref->aliasname, NAMEDATALEN);
 	}
 	else
 		extraInfo[0] = '\0';
