@@ -4,14 +4,13 @@
  * 30159 Hannover, Germany
  */
 
-#include <stdio.h>
+#include "postgres.h"
+
 #include <errno.h>
-#include <sys/types.h>
 #include "sys/ldr.h"
 #include <a.out.h>
 #include "ldfcn.h"
 
-#include "postgres.h"
 #include "dynloader.h"
 
 #ifndef HAVE_DLOPEN
