@@ -3,12 +3,12 @@
  * dynloader.c
  *	  dynamic loader for HP-UX using the shared library mechanism
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/port/dynloader/hpux.c,v 1.19 2001/10/25 05:49:40 momjian Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/port/dynloader/hpux.c,v 1.22.4.1 2004/01/04 04:06:45 tgl Exp $
  *
  *	NOTES
  *		all functions are defined here -- it's impossible to trace the
@@ -16,11 +16,11 @@
  *
  *-------------------------------------------------------------------------
  */
+#include "postgres.h"
+
 /* System includes */
 #include <a.out.h>
 
-#include "postgres.h"
-
 #include "dl.h"
 #include "dynloader.h"
 #include "utils/dynamic_loader.h"
@@ -57,7 +57,7 @@ pg_dlclose(void *handle)
 }
 
 char *
-pg_dlerror()
+pg_dlerror(void)
 {
 	static char errmsg[] = "shl_load failed";
 
