@@ -3,12 +3,12 @@
  * sinval.c
  *	  POSTGRES shared cache invalidation communication code.
  *
- * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/storage/ipc/sinval.c,v 1.52.2.1 2002/11/21 06:36:27 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/storage/ipc/sinval.c,v 1.61 2003/10/01 21:30:52 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -47,9 +47,11 @@ InitBackendSharedInvalidationState(void)
 	flag = SIBackendInit(shmInvalBuffer);
 	LWLockRelease(SInvalLock);
 	if (flag < 0)				/* unexpected problem */
-		elog(FATAL, "Backend cache invalidation initialization failed");
+		elog(FATAL, "shared cache invalidation initialization failed");
 	if (flag == 0)				/* expected problem: MaxBackends exceeded */
-		elog(FATAL, "Sorry, too many clients already");
+		ereport(FATAL,
+				(errcode(ERRCODE_TOO_MANY_CONNECTIONS),
+				 errmsg("sorry, too many clients already")));
 }
 
 /*
@@ -65,12 +67,18 @@ SendSharedInvalidMessage(SharedInvalidationMessage *msg)
 	insertOK = SIInsertDataEntry(shmInvalBuffer, msg);
 	LWLockRelease(SInvalLock);
 	if (!insertOK)
-		elog(DEBUG3, "SendSharedInvalidMessage: SI buffer overflow");
+		elog(DEBUG4, "SI buffer overflow");
 }
 
 /*
  * ReceiveSharedInvalidMessages
  *		Process shared-cache-invalidation messages waiting for this backend
+ *
+ * NOTE: it is entirely possible for this routine to be invoked recursively
+ * as a consequence of processing inside the invalFunction or resetFunction.
+ * Hence, we must be holding no SI resources when we call them.  The only
+ * bad side-effect is that SIDelExpiredDataEntries might be called extra
+ * times on the way out of a nested call.
  */
 void
 ReceiveSharedInvalidMessages(
@@ -108,7 +116,7 @@ ReceiveSharedInvalidMessages(
 		if (getResult < 0)
 		{
 			/* got a reset message */
-			elog(DEBUG3, "ReceiveSharedInvalidMessages: cache state reset");
+			elog(DEBUG4, "cache state reset");
 			resetFunction();
 		}
 		else
@@ -298,15 +306,15 @@ GetOldestXmin(bool allDbs)
  * This ensures that the set of transactions seen as "running" by the
  * current xact will not change after it takes the snapshot.
  *
- * Also, we compute the current global xmin (oldest xmin across all running
+ * We also compute the current global xmin (oldest xmin across all running
  * transactions) and save it in RecentGlobalXmin.  This is the same
- * computation done by GetOldestXmin(TRUE).
+ * computation done by GetOldestXmin(TRUE).  The xmin value is also stored
+ * into RecentXmin.
  *----------
  */
 Snapshot
-GetSnapshotData(bool serializable)
+GetSnapshotData(Snapshot snapshot, bool serializable)
 {
-	Snapshot	snapshot = (Snapshot) malloc(sizeof(SnapshotData));
 	SISeg	   *segP = shmInvalBuffer;
 	ProcState  *stateP = segP->procState;
 	TransactionId xmin;
@@ -315,18 +323,30 @@ GetSnapshotData(bool serializable)
 	int			index;
 	int			count = 0;
 
-	if (snapshot == NULL)
-		elog(ERROR, "Memory exhausted in GetSnapshotData");
+	Assert(snapshot != NULL);
 
 	/*
 	 * Allocating space for MaxBackends xids is usually overkill;
 	 * lastBackend would be sufficient.  But it seems better to do the
 	 * malloc while not holding the lock, so we can't look at lastBackend.
+	 *
+	 * This does open a possibility for avoiding repeated malloc/free:
+	 * since MaxBackends does not change at runtime, we can simply reuse
+	 * the previous xip array if any.  (This relies on the fact that all
+	 * calls pass static SnapshotData structs.)
 	 */
-	snapshot->xip = (TransactionId *)
-		malloc(MaxBackends * sizeof(TransactionId));
 	if (snapshot->xip == NULL)
-		elog(ERROR, "Memory exhausted in GetSnapshotData");
+	{
+		/*
+		 * First call for this snapshot
+		 */
+		snapshot->xip = (TransactionId *)
+			malloc(MaxBackends * sizeof(TransactionId));
+		if (snapshot->xip == NULL)
+			ereport(ERROR,
+					(errcode(ERRCODE_OUT_OF_MEMORY),
+					 errmsg("out of memory")));
+	}
 
 	globalxmin = xmin = GetCurrentTransactionId();
 
@@ -419,7 +439,9 @@ GetSnapshotData(bool serializable)
 	if (TransactionIdPrecedes(xmin, globalxmin))
 		globalxmin = xmin;
 
+	/* Update globals for use by VACUUM */
 	RecentGlobalXmin = globalxmin;
+	RecentXmin = xmin;
 
 	snapshot->xmin = xmin;
 	snapshot->xmax = xmax;
