@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/utils/adt/arrayfuncs.c,v 1.100 2003/09/25 06:58:03 petere Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/utils/adt/arrayfuncs.c,v 1.100.2.1 2004/06/08 20:28:29 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -1841,6 +1841,12 @@ array_set_slice(ArrayType *array,
 			lb[i] = lowerIndx[i];
 		}
 
+		/* complain if too few source items; we ignore extras, however */
+		if (nelems < ArrayGetNItems(nSubscripts, dim))
+			ereport(ERROR,
+					(errcode(ERRCODE_ARRAY_SUBSCRIPT_ERROR),
+					 errmsg("source array too small")));
+
 		return construct_md_array(dvalues, nSubscripts, dim, lb, elmtype,
 								  elmlen, elmbyval, elmalign);
 	}
