@@ -4,11 +4,11 @@
  *	  Display type names "nicely".
  *
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/utils/adt/format_type.c,v 1.23 2001/11/19 19:51:20 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/utils/adt/format_type.c,v 1.36 2003/08/04 02:40:04 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -17,25 +17,23 @@
 
 #include <ctype.h>
 
-#include "fmgr.h"
+#include "catalog/namespace.h"
 #include "catalog/pg_type.h"
+#include "fmgr.h"
 #include "utils/builtins.h"
 #include "utils/datetime.h"
 #include "utils/numeric.h"
+#include "utils/lsyscache.h"
 #include "utils/syscache.h"
-#ifdef MULTIBYTE
 #include "mb/pg_wchar.h"
-#endif
-
-
-#define MASK(b) (1 << (b))
 
 #define MAX_INT32_LEN 11
 #define _textin(str) DirectFunctionCall1(textin, CStringGetDatum(str))
 
 static char *format_type_internal(Oid type_oid, int32 typemod,
-								  bool typemod_given, bool allow_invalid);
-static char *psnprintf(size_t len, const char *fmt, ...)
+					 bool typemod_given, bool allow_invalid);
+static char *
+psnprintf(size_t len, const char *fmt,...)
 /* This lets gcc check the format string for consistency. */
 __attribute__((format(printf, 2, 3)));
 
@@ -47,17 +45,17 @@ __attribute__((format(printf, 2, 3)));
  * pg_attribute.atttypmod. This function will get the type name and
  * format it and the modifier to canonical SQL format, if the type is
  * a standard type. Otherwise you just get pg_type.typname back,
- * double quoted if it contains funny characters.
+ * double quoted if it contains funny characters or matches a keyword.
  *
  * If typemod is NULL then we are formatting a type name in a context where
- * no typemod is available, eg a function argument or result type.  This
+ * no typemod is available, eg a function argument or result type.	This
  * yields a slightly different result from specifying typemod = -1 in some
  * cases.  Given typemod = -1 we feel compelled to produce an output that
  * the parser will interpret as having typemod -1, so that pg_dump will
- * produce CREATE TABLE commands that recreate the original state.  But
+ * produce CREATE TABLE commands that recreate the original state.	But
  * given NULL typemod, we assume that the parser's interpretation of
  * typemod doesn't matter, and so we are willing to output a slightly
- * "prettier" representation of the same type.  For example, type = bpchar
+ * "prettier" representation of the same type.	For example, type = bpchar
  * and typemod = NULL gets you "character", whereas typemod = -1 gets you
  * "bpchar" --- the former will be interpreted as character(1) by the
  * parser, which does not yield typemod -1.
@@ -80,9 +78,7 @@ format_type(PG_FUNCTION_ARGS)
 	type_oid = PG_GETARG_OID(0);
 
 	if (PG_ARGISNULL(1))
-	{
 		result = format_type_internal(type_oid, -1, false, true);
-	}
 	else
 	{
 		typemod = PG_GETARG_INT32(1);
@@ -121,10 +117,9 @@ format_type_internal(Oid type_oid, int32 typemod,
 {
 	bool		with_typemod = typemod_given && (typemod >= 0);
 	HeapTuple	tuple;
+	Form_pg_type typeform;
 	Oid			array_base_type;
-	int16		typlen;
 	bool		is_array;
-	char	   *name;
 	char	   *buf;
 
 	if (type_oid == InvalidOid && allow_invalid)
@@ -138,13 +133,20 @@ format_type_internal(Oid type_oid, int32 typemod,
 		if (allow_invalid)
 			return pstrdup("???");
 		else
-			elog(ERROR, "could not locate data type with oid %u in catalog",
-				 type_oid);
+			elog(ERROR, "cache lookup failed for type %u", type_oid);
 	}
+	typeform = (Form_pg_type) GETSTRUCT(tuple);
 
-	array_base_type = ((Form_pg_type) GETSTRUCT(tuple))->typelem;
-	typlen = ((Form_pg_type) GETSTRUCT(tuple))->typlen;
-	if (array_base_type != InvalidOid && typlen < 0)
+	/*
+	 * Check if it's an array (and not a domain --- we don't want to show
+	 * the substructure of a domain type).	Fixed-length array types such
+	 * as "name" shouldn't get deconstructed either.
+	 */
+	array_base_type = typeform->typelem;
+
+	if (array_base_type != InvalidOid &&
+		typeform->typlen == -1 &&
+		typeform->typtype != 'd')
 	{
 		/* Switch our attention to the array element type */
 		ReleaseSysCache(tuple);
@@ -156,15 +158,28 @@ format_type_internal(Oid type_oid, int32 typemod,
 			if (allow_invalid)
 				return pstrdup("???[]");
 			else
-				elog(ERROR, "could not locate data type with oid %u in catalog",
-					 type_oid);
+				elog(ERROR, "cache lookup failed for type %u", type_oid);
 		}
-		is_array = true;
+		typeform = (Form_pg_type) GETSTRUCT(tuple);
 		type_oid = array_base_type;
+		is_array = true;
 	}
 	else
 		is_array = false;
 
+	/*
+	 * See if we want to special-case the output for certain built-in
+	 * types. Note that these special cases should all correspond to
+	 * special productions in gram.y, to ensure that the type name will be
+	 * taken as a system type, not a user type of the same name.
+	 *
+	 * If we do not provide a special-case output here, the type name will be
+	 * handled the same way as a user type name --- in particular, it will
+	 * be double-quoted if it matches any lexer keyword.  This behavior is
+	 * essential for some cases, such as types "bit" and "char".
+	 */
+	buf = NULL;					/* flag for no special case */
+
 	switch (type_oid)
 	{
 		case BITOID:
@@ -178,7 +193,6 @@ format_type_internal(Oid type_oid, int32 typemod,
 				 * BIT(1) per SQL spec.  Report it as the quoted typename
 				 * so that parser will not assign a bogus typmod.
 				 */
-				buf = pstrdup("\"bit\"");
 			}
 			else
 				buf = pstrdup("bit");
@@ -199,20 +213,11 @@ format_type_internal(Oid type_oid, int32 typemod,
 				 * which means CHARACTER(1) per SQL spec.  Report it as
 				 * bpchar so that parser will not assign a bogus typmod.
 				 */
-				buf = pstrdup("bpchar");
 			}
 			else
 				buf = pstrdup("character");
 			break;
 
-		case CHAROID:
-			/*
-			 * This char type is the single-byte version. You have to
-			 * double-quote it to get at it in the parser.
-			 */
-			buf = pstrdup("\"char\"");
-			break;
-
 		case FLOAT4OID:
 			buf = pstrdup("real");
 			break;
@@ -245,60 +250,71 @@ format_type_internal(Oid type_oid, int32 typemod,
 		case INTERVALOID:
 			if (with_typemod)
 			{
-				int			fields = typemod >> 16;
-				int			precision = typemod & 0xFFFF;
+				int			fields = INTERVAL_RANGE(typemod);
+				int			precision = INTERVAL_PRECISION(typemod);
 				const char *fieldstr;
 
 				switch (fields)
 				{
-					case MASK(YEAR):
+					case INTERVAL_MASK(YEAR):
 						fieldstr = " year";
 						break;
-					case MASK(MONTH):
+					case INTERVAL_MASK(MONTH):
 						fieldstr = " month";
 						break;
-					case MASK(DAY):
+					case INTERVAL_MASK(DAY):
 						fieldstr = " day";
 						break;
-					case MASK(HOUR):
+					case INTERVAL_MASK(HOUR):
 						fieldstr = " hour";
 						break;
-					case MASK(MINUTE):
+					case INTERVAL_MASK(MINUTE):
 						fieldstr = " minute";
 						break;
-					case MASK(SECOND):
+					case INTERVAL_MASK(SECOND):
 						fieldstr = " second";
 						break;
-					case MASK(YEAR) | MASK(MONTH):
+						case INTERVAL_MASK(YEAR)
+					| INTERVAL_MASK(MONTH):
 						fieldstr = " year to month";
 						break;
-					case MASK(DAY) | MASK(HOUR):
+						case INTERVAL_MASK(DAY)
+					| INTERVAL_MASK(HOUR):
 						fieldstr = " day to hour";
 						break;
-					case MASK(DAY) | MASK(HOUR) | MASK(MINUTE):
+						case INTERVAL_MASK(DAY)
+							| INTERVAL_MASK(HOUR)
+					| INTERVAL_MASK(MINUTE):
 						fieldstr = " day to minute";
 						break;
-					case MASK(DAY) | MASK(HOUR) | MASK(MINUTE) | MASK(SECOND):
+						case INTERVAL_MASK(DAY)
+							| INTERVAL_MASK(HOUR)
+							| INTERVAL_MASK(MINUTE)
+					| INTERVAL_MASK(SECOND):
 						fieldstr = " day to second";
 						break;
-					case MASK(HOUR) | MASK(MINUTE):
+						case INTERVAL_MASK(HOUR)
+					| INTERVAL_MASK(MINUTE):
 						fieldstr = " hour to minute";
 						break;
-					case MASK(HOUR) | MASK(MINUTE) | MASK(SECOND):
+						case INTERVAL_MASK(HOUR)
+							| INTERVAL_MASK(MINUTE)
+					| INTERVAL_MASK(SECOND):
 						fieldstr = " hour to second";
 						break;
-					case MASK(MINUTE) | MASK(SECOND):
+						case INTERVAL_MASK(MINUTE)
+					| INTERVAL_MASK(SECOND):
 						fieldstr = " minute to second";
 						break;
-					case 0x7FFF:
+					case INTERVAL_FULL_RANGE:
 						fieldstr = "";
 						break;
 					default:
-						elog(DEBUG, "Invalid INTERVAL typmod 0x%x", typemod);
+						elog(ERROR, "invalid INTERVAL typmod: 0x%x", typemod);
 						fieldstr = "";
 						break;
 				}
-				if (precision != 0xFFFF)
+				if (precision != INTERVAL_FULL_PRECISION)
 					buf = psnprintf(100, "interval(%d)%s",
 									precision, fieldstr);
 				else
@@ -357,19 +373,27 @@ format_type_internal(Oid type_oid, int32 typemod,
 			else
 				buf = pstrdup("character varying");
 			break;
+	}
 
-		default:
-			name = NameStr(((Form_pg_type) GETSTRUCT(tuple))->typname);
-			/*
-			 * Double-quote the name if it's not a standard identifier.
-			 * Note this is *necessary* for ruleutils.c's use.
-			 */
-			if (strspn(name, "abcdefghijklmnopqrstuvwxyz0123456789_") != strlen(name)
-				|| isdigit((unsigned char) name[0]))
-				buf = psnprintf(strlen(name) + 3, "\"%s\"", name);
-			else
-				buf = pstrdup(name);
-			break;
+	if (buf == NULL)
+	{
+		/*
+		 * Default handling: report the name as it appears in the catalog.
+		 * Here, we must qualify the name if it is not visible in the
+		 * search path, and we must double-quote it if it's not a standard
+		 * identifier or if it matches any keyword.
+		 */
+		char	   *nspname;
+		char	   *typname;
+
+		if (TypeIsVisible(type_oid))
+			nspname = NULL;
+		else
+			nspname = get_namespace_name(typeform->typnamespace);
+
+		typname = NameStr(typeform->typname);
+
+		buf = quote_qualified_identifier(nspname, typname);
 	}
 
 	if (is_array)
@@ -382,11 +406,11 @@ format_type_internal(Oid type_oid, int32 typemod,
 
 
 /*
- * type_maximum_size --- determine maximum width of a varlena column
+ * type_maximum_size --- determine maximum width of a variable-width column
  *
  * If the max width is indeterminate, return -1.  In particular, we return
  * -1 for any type not known to this routine.  We assume the caller has
- * already determined that the type is a varlena type, so it's not
+ * already determined that the type is a variable-width type, so it's not
  * necessary to look up the type's pg_type tuple here.
  *
  * This may appear unrelated to format_type(), but in fact the two routines
@@ -404,14 +428,11 @@ type_maximum_size(Oid type_oid, int32 typemod)
 		case BPCHAROID:
 		case VARCHAROID:
 			/* typemod includes varlena header */
-#ifdef MULTIBYTE
+
 			/* typemod is in characters not bytes */
 			return (typemod - VARHDRSZ) *
 				pg_encoding_max_length(GetDatabaseEncoding())
 				+ VARHDRSZ;
-#else
-			return typemod;
-#endif
 
 		case NUMERICOID:
 			/* precision (ie, max # of digits) is in upper bits of typmod */
@@ -494,7 +515,7 @@ oidvectortypes(PG_FUNCTION_ARGS)
 
 /* snprintf into a palloc'd string */
 static char *
-psnprintf(size_t len, const char *fmt, ...)
+psnprintf(size_t len, const char *fmt,...)
 {
 	va_list		ap;
 	char	   *buf;
