@@ -70,11 +70,11 @@
  *	they need not live beyond the end of the current transaction.
  *
  *
- * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/utils/cache/inval.c,v 1.55 2002/09/04 20:31:29 momjian Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/utils/cache/inval.c,v 1.58 2003/08/04 02:40:06 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -449,10 +449,7 @@ LocalExecuteInvalidationMessage(SharedInvalidationMessage *msg)
 		}
 	}
 	else
-	{
-		elog(FATAL, "ExecuteInvalidationMessage: bogus message id %d",
-			 msg->id);
-	}
+		elog(FATAL, "unrecognized SI message id: %d", msg->id);
 }
 
 /*
@@ -705,7 +702,7 @@ CacheRegisterSyscacheCallback(int cacheid,
 							  Datum arg)
 {
 	if (cache_callback_count >= MAX_CACHE_CALLBACKS)
-		elog(FATAL, "Out of cache_callback_list slots");
+		elog(FATAL, "out of cache_callback_list slots");
 
 	cache_callback_list[cache_callback_count].id = cacheid;
 	cache_callback_list[cache_callback_count].function = func;
@@ -728,7 +725,7 @@ CacheRegisterRelcacheCallback(CacheCallbackFunction func,
 							  Datum arg)
 {
 	if (cache_callback_count >= MAX_CACHE_CALLBACKS)
-		elog(FATAL, "Out of cache_callback_list slots");
+		elog(FATAL, "out of cache_callback_list slots");
 
 	cache_callback_list[cache_callback_count].id = SHAREDINVALRELCACHE_ID;
 	cache_callback_list[cache_callback_count].function = func;
