@@ -5,7 +5,7 @@
  *	Implements the basic DB functions used by the archiver.
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/bin/pg_dump/pg_backup_db.c,v 1.43.2.1 2003/02/01 22:07:14 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/bin/pg_dump/pg_backup_db.c,v 1.50 2003/10/03 20:10:59 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -45,7 +45,7 @@ _parse_version(ArchiveHandle *AH, const char *versionString)
 
 	v = parse_version(versionString);
 	if (v < 0)
-		die_horribly(AH, modulename, "unable to parse version string \"%s\"\n", versionString);
+		die_horribly(AH, modulename, "could not parse version string \"%s\"\n", versionString);
 
 	return v;
 }
@@ -53,40 +53,23 @@ _parse_version(ArchiveHandle *AH, const char *versionString)
 static void
 _check_database_version(ArchiveHandle *AH, bool ignoreVersion)
 {
-	PGresult   *res;
 	int			myversion;
 	const char *remoteversion_str;
 	int			remoteversion;
-	PGconn	   *conn = AH->connection;
 
 	myversion = _parse_version(AH, PG_VERSION);
 
-	/*
-	 *	Autocommit could be off.  We turn it off later but we have to check
-	 *	the database version first.
-	 */
+	remoteversion_str = PQparameterStatus(AH->connection, "server_version");
+	if (!remoteversion_str)
+		die_horribly(AH, modulename, "could not get server_version from libpq\n");
 
-	res = PQexec(conn, "BEGIN;SELECT version();");
-	if (!res ||
-		PQresultStatus(res) != PGRES_TUPLES_OK ||
-		PQntuples(res) != 1)
-		die_horribly(AH, modulename, "could not get version from server: %s", PQerrorMessage(conn));
-
-	remoteversion_str = PQgetvalue(res, 0, 0);
-	remoteversion = _parse_version(AH, remoteversion_str + 11);
-
-	PQclear(res);
-
-	res = PQexec(conn, "COMMIT;");
-	if (!res ||
-		PQresultStatus(res) != PGRES_COMMAND_OK)
-		die_horribly(AH, modulename, "could not get version from server: %s", PQerrorMessage(conn));
-	PQclear(res);
+	remoteversion = _parse_version(AH, remoteversion_str);
 
 	AH->public.remoteVersion = remoteversion;
 
 	if (myversion != remoteversion
-		&& (remoteversion < AH->public.minRemoteVersion || remoteversion > AH->public.maxRemoteVersion))
+		&& (remoteversion < AH->public.minRemoteVersion ||
+			remoteversion > AH->public.maxRemoteVersion))
 	{
 		write_msg(NULL, "server version: %s; %s version: %s\n",
 				  remoteversion_str, progname, PG_VERSION);
@@ -133,11 +116,6 @@ ReconnectToServer(ArchiveHandle *AH, const char *dbname, const char *username)
 	PQfinish(AH->connection);
 	AH->connection = newConn;
 
-	/* don't assume we still know the output schema */
-	if (AH->currSchema)
-		free(AH->currSchema);
-	AH->currSchema = strdup("");
-
 	return 1;
 }
 
@@ -165,7 +143,7 @@ _connectDB(ArchiveHandle *AH, const char *reqdb, const char *requser)
 	else
 		newuser = (char *) requser;
 
-	ahlog(AH, 1, "connecting to database %s as user %s\n", newdb, newuser);
+	ahlog(AH, 1, "connecting to database \"%s\" as user \"%s\"\n", newdb, newuser);
 
 	if (AH->requirePassword)
 	{
@@ -213,6 +191,9 @@ _connectDB(ArchiveHandle *AH, const char *reqdb, const char *requser)
 	if (password)
 		free(password);
 
+	/* check for version mismatch */
+	_check_database_version(AH, true);
+
 	PQsetNoticeProcessor(newConn, notice_processor, NULL);
 
 	return newConn;
@@ -286,18 +267,6 @@ ConnectDatabase(Archive *AHX,
 	/* check for version mismatch */
 	_check_database_version(AH, ignoreVersion);
 
-	/* Turn autocommit on */
-	if (AH->public.remoteVersion >= 70300)
-	{
-		PGresult   *res;
-
-		res = PQexec(AH->connection, "SET autocommit TO 'on'");
-		if (!res || PQresultStatus(res) != PGRES_COMMAND_OK)
-			die_horribly(AH, NULL, "SET autocommit TO 'on' failed: %s",
-						  PQerrorMessage(AH->connection));
-		PQclear(res);
-	}
-
 	PQsetNoticeProcessor(AH->connection, notice_processor, NULL);
 
 	return AH->connection;
@@ -611,7 +580,7 @@ FixupBlobRefs(ArchiveHandle *AH, TocEntry *te)
 
 	res = PQexec(AH->blobConnection, tblQry->data);
 	if (!res)
-		die_horribly(AH, modulename, "could not find oid columns of table \"%s\": %s",
+		die_horribly(AH, modulename, "could not find OID columns of table \"%s\": %s",
 					 te->tag, PQerrorMessage(AH->connection));
 
 	if ((n = PQntuples(res)) == 0)
