@@ -1,9 +1,9 @@
 /*
  * psql - the PostgreSQL interactive terminal
  *
- * Copyright 2000-2002 by PostgreSQL Global Development Group
+ * Copyright (c) 2000-2003, PostgreSQL Global Development Group
  *
- * $Header: /cvsroot/pgsql/src/bin/psql/command.c,v 1.84.2.1 2003/01/24 05:23:55 tgl Exp $
+ * $Header: /cvsroot/pgsql-server/src/bin/psql/command.c,v 1.105 2003/10/11 18:04:26 momjian Exp $
  */
 #include "postgres_fe.h"
 #include "command.h"
@@ -66,8 +66,6 @@ static bool do_edit(const char *filename_arg, PQExpBuffer query_buf);
 static bool do_connect(const char *new_dbname, const char *new_user);
 static bool do_shell(const char *command);
 
-
-
 /*----------
  * HandleSlashCmds:
  *
@@ -305,7 +303,7 @@ exec_command(const char *cmd,
 
 		if (chdir(dir) == -1)
 		{
-			psql_error("\\%s: could not change directory to '%s': %s\n",
+			psql_error("\\%s: could not change directory to \"%s\": %s\n",
 					   cmd, dir, strerror(errno));
 			success = false;
 		}
@@ -350,15 +348,27 @@ exec_command(const char *cmd,
 			case 'a':
 				success = describeAggregates(pattern, show_verbose);
 				break;
+			case 'c':
+				success = listConversions(pattern);
+				break;
+			case 'C':
+				success = listCasts(pattern);
+				break;
 			case 'd':
 				success = objectDescription(pattern);
 				break;
+			case 'D':
+				success = listDomains(pattern);
+				break;
 			case 'f':
 				success = describeFunctions(pattern, show_verbose);
 				break;
 			case 'l':
 				success = do_lo_list();
 				break;
+			case 'n':
+				success = listSchemas(pattern);
+				break;
 			case 'o':
 				success = describeOperators(pattern);
 				break;
@@ -378,9 +388,6 @@ exec_command(const char *cmd,
 			case 'u':
 				success = describeUsers(pattern);
 				break;
-			case 'D':
-				success = listDomains(pattern);
-				break;
 
 			default:
 				status = CMD_UNKNOWN;
@@ -450,19 +457,22 @@ exec_command(const char *cmd,
 		char	   *encoding = scan_option(&string, OT_NORMAL, NULL, false);
 
 		if (!encoding)
+		{
 			/* show encoding */
 			puts(pg_encoding_to_char(pset.encoding));
+		}
 		else
 		{
 			/* set encoding */
 			if (PQsetClientEncoding(pset.db, encoding) == -1)
 				psql_error("%s: invalid encoding name or conversion procedure not found\n", encoding);
-
 			else
 			{
 				/* save encoding info into psql internal data */
 				pset.encoding = PQclientEncoding(pset.db);
-				SetVariable(pset.vars, "ENCODING", pg_encoding_to_char(pset.encoding));
+				pset.popt.topt.encoding = pset.encoding;
+				SetVariable(pset.vars, "ENCODING",
+							pg_encoding_to_char(pset.encoding));
 			}
 			free(encoding);
 		}
@@ -647,7 +657,7 @@ exec_command(const char *cmd,
 		success = saveHistory(fname ? fname : "/dev/tty");
 
 		if (success && !quiet && fname)
-			printf(gettext("Wrote history to %s.\n"), fname);
+			printf(gettext("Wrote history to file \"%s\".\n"), fname);
 		free(fname);
 	}
 
@@ -659,15 +669,7 @@ exec_command(const char *cmd,
 		if (!opt0)
 		{
 			/* list all variables */
-
-			/*
-			 * XXX This is in utter violation of the GetVariable
-			 * abstraction, but I have not bothered to do it better.
-			 */
-			struct _variable *ptr;
-
-			for (ptr = pset.vars; ptr->next; ptr = ptr->next)
-				fprintf(stdout, "%s = '%s'\n", ptr->next->name, ptr->next->value);
+			PrintVariables(pset.vars);
 			success = true;
 		}
 		else
@@ -694,7 +696,13 @@ exec_command(const char *cmd,
 				free(opt);
 			}
 
-			if (!SetVariable(pset.vars, opt0, newval))
+			if (SetVariable(pset.vars, opt0, newval))
+			{
+				/* Check for special variables */
+				if (strcmp(opt0, "VERBOSITY") == 0)
+					SyncVerbosityVariable();
+			}
+			else
 			{
 				psql_error("\\%s: error\n", cmd);
 				success = false;
@@ -725,12 +733,9 @@ exec_command(const char *cmd,
 		if (!quiet)
 		{
 			if (pset.timing)
-				puts(gettext(("Timing is on.")));
+				puts(gettext("Timing is on."));
 			else
-			{
-				puts(gettext(("Timing is off.")));
-
-			}
+				puts(gettext("Timing is off."));
 		}
 	}
 
@@ -871,7 +876,9 @@ exec_command(const char *cmd,
 	while ((val = scan_option(&string, OT_NORMAL, NULL, false)))
 	{
 		if (status != CMD_UNKNOWN)
-			psql_error("\\%s: extra argument '%s' ignored\n", cmd, val);
+			psql_error("\\%s: extra argument \"%s\" ignored\n", cmd, val);
+		if (val)
+			free(val);
 	}
 
 	if (options_string && continue_parse)
@@ -1061,9 +1068,7 @@ scan_option(char **string, enum option_type type, char *quote, bool semicolon)
 				save_char = options_string[pos + token_end + 1];
 				options_string[pos + token_end + 1] = '\0';
 				value = GetVariable(pset.vars, options_string + pos + 1);
-				if (!value)
-					value = "";
-				return_val = xstrdup(value);
+				return_val = xstrdup(value ? value : "");
 				options_string[pos + token_end + 1] = save_char;
 				*string = &options_string[pos + token_end + 1];
 				/* XXX should we set *quote to ':' here? */
@@ -1275,15 +1280,9 @@ unescape(const unsigned char *source, size_t len)
 				case '7':
 				case '8':
 				case '9':
-					{
-						long int	l;
-						char	   *end;
+					c = parse_char((char **) &p);
+					break;
 
-						l = strtol(p, &end, 0);
-						c = (char) l;
-						p = end - 1;
-						break;
-					}
 				default:
 					c = *p;
 			}
@@ -1333,11 +1332,7 @@ do_connect(const char *new_dbname, const char *new_user)
 	bool		success = false;
 
 	/* Delete variables (in case we fail before setting them anew) */
-	SetVariable(pset.vars, "DBNAME", NULL);
-	SetVariable(pset.vars, "USER", NULL);
-	SetVariable(pset.vars, "HOST", NULL);
-	SetVariable(pset.vars, "PORT", NULL);
-	SetVariable(pset.vars, "ENCODING", NULL);
+	UnsyncVariables();
 
 	/* If dbname is "" then use old name, else new one (even if NULL) */
 	if (oldconn && new_dbname && PQdb(oldconn) && strcmp(new_dbname, "") == 0)
@@ -1419,12 +1414,12 @@ do_connect(const char *new_dbname, const char *new_user)
 		if (!QUIET())
 		{
 			if (userparam != new_user)	/* no new user */
-				printf(gettext("You are now connected to database %s.\n"), dbparam);
+				printf(gettext("You are now connected to database \"%s\".\n"), dbparam);
 			else if (dbparam != new_dbname)		/* no new db */
-				printf(gettext("You are now connected as new user %s.\n"), new_user);
+				printf(gettext("You are now connected as new user \"%s\".\n"), new_user);
 			else
 /* both new */
-				printf(gettext("You are now connected to database %s as user %s.\n"),
+				printf(gettext("You are now connected to database \"%s\" as user \"%s\".\n"),
 					   PQdb(pset.db), PQuser(pset.db));
 		}
 
@@ -1435,54 +1430,75 @@ do_connect(const char *new_dbname, const char *new_user)
 	}
 
 	PQsetNoticeProcessor(pset.db, NoticeProcessor, NULL);
-	pset.encoding = PQclientEncoding(pset.db);
 
 	/* Update variables */
+	SyncVariables();
+
+	return success;
+}
+
+
+/*
+ * SyncVariables
+ *
+ * Make psql's internal variables agree with connection state upon
+ * establishing a new connection.
+ */
+void
+SyncVariables(void)
+{
+	/* get stuff from connection */
+	pset.encoding = PQclientEncoding(pset.db);
+	pset.popt.topt.encoding = pset.encoding;
+
 	SetVariable(pset.vars, "DBNAME", PQdb(pset.db));
 	SetVariable(pset.vars, "USER", PQuser(pset.db));
 	SetVariable(pset.vars, "HOST", PQhost(pset.db));
 	SetVariable(pset.vars, "PORT", PQport(pset.db));
 	SetVariable(pset.vars, "ENCODING", pg_encoding_to_char(pset.encoding));
 
-	pset.issuper = test_superuser(PQuser(pset.db));
-
-	return success;
+	/* send stuff to it, too */
+	SyncVerbosityVariable();
 }
 
-
-
 /*
- * Test if the given user is a database superuser.
- * (Is used to set up the prompt right.)
+ * UnsyncVariables
+ *
+ * Clear variables that should be not be set when there is no connection.
  */
-bool
-test_superuser(const char *username)
+void
+UnsyncVariables(void)
 {
-	PGresult   *res;
-	PQExpBufferData buf;
-	bool		answer;
-
-	if (!username)
-		return false;
-
-	/*
-	 * Use begin/commit to avoid starting a transaction block if server
-	 * has autocommit off by default.
-	 */
-	initPQExpBuffer(&buf);
-	printfPQExpBuffer(&buf, "BEGIN; SELECT usesuper FROM pg_catalog.pg_user WHERE usename = '%s'; COMMIT", username);
-	res = PSQLexec(buf.data, true);
-	termPQExpBuffer(&buf);
-
-	answer =
-		(res && PQntuples(res) > 0 && PQnfields(res) > 0
-		 && !PQgetisnull(res, 0, 0)
-		 && PQgetvalue(res, 0, 0)
-		 && strcmp(PQgetvalue(res, 0, 0), "t") == 0);
-	PQclear(res);
-	return answer;
+	SetVariable(pset.vars, "DBNAME", NULL);
+	SetVariable(pset.vars, "USER", NULL);
+	SetVariable(pset.vars, "HOST", NULL);
+	SetVariable(pset.vars, "PORT", NULL);
+	SetVariable(pset.vars, "ENCODING", NULL);
 }
 
+/*
+ * Update connection state from VERBOSITY variable
+ */
+void
+SyncVerbosityVariable(void)
+{
+	switch (SwitchVariable(pset.vars, "VERBOSITY",
+						   "default", "terse", "verbose", NULL))
+	{
+		case 1:			/* default */
+			PQsetErrorVerbosity(pset.db, PQERRORS_DEFAULT);
+			break;
+		case 2:			/* terse */
+			PQsetErrorVerbosity(pset.db, PQERRORS_TERSE);
+			break;
+		case 3:			/* verbose */
+			PQsetErrorVerbosity(pset.db, PQERRORS_VERBOSE);
+			break;
+		default:				/* not set or unrecognized value */
+			PQsetErrorVerbosity(pset.db, PQERRORS_DEFAULT);
+			break;
+	}
+}
 
 
 /*
@@ -1518,10 +1534,14 @@ editFile(const char *fname)
 	sys = malloc(strlen(editorName) + strlen(fname) + 10 + 1);
 	if (!sys)
 		return false;
-	sprintf(sys, "exec  %s '%s'", editorName, fname);
+	sprintf(sys,
+#ifndef WIN32
+			"exec "
+#endif
+			"%s '%s'", editorName, fname);
 	result = system(sys);
 	if (result == -1)
-		psql_error("could not start editor %s\n", editorName);
+		psql_error("could not start editor \"%s\"\n", editorName);
 	else if (result == 127)
 		psql_error("could not start /bin/sh\n");
 	free(sys);
@@ -1568,7 +1588,7 @@ do_edit(const char *filename_arg, PQExpBuffer query_buf)
 
 		if (fd == -1 || !stream)
 		{
-			psql_error("could not open temporary file %s: %s\n", fname, strerror(errno));
+			psql_error("could not open temporary file \"%s\": %s\n", fname, strerror(errno));
 			error = true;
 		}
 		else
@@ -1639,6 +1659,14 @@ do_edit(const char *filename_arg, PQExpBuffer query_buf)
 				error = true;
 			}
 
+#ifdef USE_READLINE
+#ifdef HAVE_REPLACE_HISTORY_ENTRY
+
+			replace_history_entry(where_history(), query_buf->data, NULL);
+#else
+			add_history(query_buf->data);
+#endif
+#endif
 			fclose(stream);
 		}
 
@@ -1789,7 +1817,7 @@ do_pset(const char *param, const char *value, printQueryOpt *popt, bool quiet)
 			popt->nullPrint = xstrdup(value);
 		}
 		if (!quiet)
-			printf(gettext("Null display is '%s'.\n"), popt->nullPrint ? popt->nullPrint : "");
+			printf(gettext("Null display is \"%s\".\n"), popt->nullPrint ? popt->nullPrint : "");
 	}
 
 	/* field separator for unaligned text */
@@ -1801,7 +1829,7 @@ do_pset(const char *param, const char *value, printQueryOpt *popt, bool quiet)
 			popt->topt.fieldSep = xstrdup(value);
 		}
 		if (!quiet)
-			printf(gettext("Field separator is '%s'.\n"), popt->topt.fieldSep);
+			printf(gettext("Field separator is \"%s\".\n"), popt->topt.fieldSep);
 	}
 
 	/* record separator for unaligned text */
@@ -1817,7 +1845,7 @@ do_pset(const char *param, const char *value, printQueryOpt *popt, bool quiet)
 			if (strcmp(popt->topt.recordSep, "\n") == 0)
 				printf(gettext("Record separator is <newline>."));
 			else
-				printf(gettext("Record separator is '%s'.\n"), popt->topt.recordSep);
+				printf(gettext("Record separator is \"%s\".\n"), popt->topt.recordSep);
 		}
 	}
 
@@ -1873,13 +1901,20 @@ do_pset(const char *param, const char *value, printQueryOpt *popt, bool quiet)
 	/* toggle use of pager */
 	else if (strcmp(param, "pager") == 0)
 	{
-		popt->topt.pager = !popt->topt.pager;
+		if (value && strcasecmp(value, "always") == 0)
+			popt->topt.pager = 2;
+		else if (popt->topt.pager == 1)
+			popt->topt.pager = 0;
+		else
+			popt->topt.pager = 1;
 		if (!quiet)
 		{
-			if (popt->topt.pager)
-				puts(gettext("Using pager is on."));
+			if (popt->topt.pager == 1)
+				puts(gettext("Pager is used for long output."));
+			else if (popt->topt.pager == 2)
+				puts(gettext("Pager is always used."));
 			else
-				puts(gettext("Using pager is off."));
+				puts(gettext("Pager usage is off."));
 		}
 	}
 
@@ -1932,7 +1967,11 @@ do_shell(const char *command)
 			else
 				exit(EXIT_FAILURE);
 		}
-		sprintf(sys, "exec %s", shellName);
+		sprintf(sys,
+#ifndef WIN32
+				"exec "
+#endif
+				"%s", shellName);
 		result = system(sys);
 		free(sys);
 	}
