@@ -1,12 +1,14 @@
 package org.postgresql.test.jdbc2;
 
-import org.postgresql.test.JDBC2Tests;
+import org.postgresql.test.TestUtil;
+import java.sql.Connection;
+import java.sql.ResultSet;
+import java.sql.Statement;
+
 import junit.framework.TestCase;
-import java.sql.*;
-import java.math.BigDecimal;
 
 /*
- * $Id: JBuilderTest.java,v 1.5 2001/11/19 22:33:39 momjian Exp $
+ * $Id: JBuilderTest.java,v 1.8 2003/05/29 04:39:48 barry Exp $
  *
  * Some simple tests to check that the required components needed for JBuilder
  * stay working
@@ -23,20 +25,20 @@ public class JBuilderTest extends TestCase
 	// Set up the fixture for this testcase: the tables for this test.
 	protected void setUp() throws Exception
 	{
-		Connection con = JDBC2Tests.openDB();
+		Connection con = TestUtil.openDB();
 
-		JDBC2Tests.createTable( con, "test_c",
-								"source text,cost money,imageid int4" );
+		TestUtil.createTable( con, "test_c",
+							  "source text,cost money,imageid int4" );
 
-		JDBC2Tests.closeDB(con);
+		TestUtil.closeDB(con);
 	}
 
 	// Tear down the fixture for this test case.
 	protected void tearDown() throws Exception
 	{
-		Connection con = JDBC2Tests.openDB();
-		JDBC2Tests.dropTable(con, "test_c");
-		JDBC2Tests.closeDB(con);
+		Connection con = TestUtil.openDB();
+		TestUtil.dropTable(con, "test_c");
+		TestUtil.closeDB(con);
 	}
 
 	/*
@@ -46,7 +48,7 @@ public class JBuilderTest extends TestCase
 	{
 		try
 		{
-			Connection con = JDBC2Tests.openDB();
+			Connection con = TestUtil.openDB();
 
 			Statement st = con.createStatement();
 			ResultSet rs = st.executeQuery("select cost from test_c");
@@ -54,13 +56,13 @@ public class JBuilderTest extends TestCase
 
 			while (rs.next())
 			{
-				double bd = rs.getDouble(1);
+				rs.getDouble(1);
 			}
 
 			rs.close();
 			st.close();
 
-			JDBC2Tests.closeDB(con);
+			TestUtil.closeDB(con);
 		}
 		catch (Exception ex)
 		{
