@@ -3,7 +3,7 @@
  *			  procedural language
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/pl/plpgsql/src/pl_handler.c,v 1.10 2001/10/25 05:50:20 momjian Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/pl/plpgsql/src/pl_handler.c,v 1.18 2003/09/28 23:37:45 tgl Exp $
  *
  *	  This software is copyrighted by Jan Wieck - Hamburg.
  *
@@ -35,13 +35,6 @@
  *
  **********************************************************************/
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <string.h>
-
 #include "plpgsql.h"
 #include "pl.tab.h"
 
@@ -51,15 +44,46 @@
 #include "utils/builtins.h"
 #include "utils/syscache.h"
 
+static int	plpgsql_firstcall = 1;
+
+static void plpgsql_init_all(void);
+
 
 /*
- * Head of list of already-compiled functions
+ * plpgsql_init()			- postmaster-startup safe initialization
+ *
+ * DO NOT make this static --- it has to be callable by preload
  */
-static PLpgSQL_function *compiled_functions = NULL;
+void
+plpgsql_init(void)
+{
+	/* Do initialization only once */
+	if (!plpgsql_firstcall)
+		return;
 
+	plpgsql_HashTableInit();
 
-static bool func_up_to_date(PLpgSQL_function * func);
+	RegisterEOXactCallback(plpgsql_eoxact, NULL);
 
+	plpgsql_firstcall = 0;
+}
+
+/*
+ * plpgsql_init_all()		- Initialize all
+ */
+static void
+plpgsql_init_all(void)
+{
+	/* Execute any postmaster-startup safe initialization */
+	if (plpgsql_firstcall)
+		plpgsql_init();
+
+	/*
+	 * Any other initialization that must be done each time a new backend
+	 * starts -- currently none
+	 */
+
+}
 
 /* ----------
  * plpgsql_call_handler
@@ -74,66 +98,26 @@ PG_FUNCTION_INFO_V1(plpgsql_call_handler);
 Datum
 plpgsql_call_handler(PG_FUNCTION_ARGS)
 {
-	bool		isTrigger = CALLED_AS_TRIGGER(fcinfo);
-	Oid			funcOid = fcinfo->flinfo->fn_oid;
 	PLpgSQL_function *func;
 	Datum		retval;
 
+	/* perform initialization */
+	plpgsql_init_all();
+
 	/*
 	 * Connect to SPI manager
 	 */
 	if (SPI_connect() != SPI_OK_CONNECT)
-		elog(ERROR, "plpgsql: cannot connect to SPI manager");
+		elog(ERROR, "SPI_connect failed");
 
-	/*
-	 * Check if we already compiled this function and saved the pointer
-	 * (ie, current FmgrInfo has been used before)
-	 */
-	func = (PLpgSQL_function *) fcinfo->flinfo->fn_extra;
-	if (func != NULL)
-	{
-		Assert(func->fn_oid == funcOid);
-
-		/*
-		 * But is the function still up to date?
-		 */
-		if (!func_up_to_date(func))
-			func = NULL;
-	}
-
-	if (func == NULL)
-	{
-		/*
-		 * Check if we already compiled this function for another caller
-		 */
-		for (func = compiled_functions; func != NULL; func = func->next)
-		{
-			if (funcOid == func->fn_oid && func_up_to_date(func))
-				break;
-		}
-
-		/*
-		 * If not, do so and add it to the compiled ones
-		 */
-		if (func == NULL)
-		{
-			func = plpgsql_compile(funcOid,
-								   isTrigger ? T_TRIGGER : T_FUNCTION);
-			func->next = compiled_functions;
-			compiled_functions = func;
-		}
-
-		/*
-		 * Save pointer in FmgrInfo to avoid search on subsequent calls
-		 */
-		fcinfo->flinfo->fn_extra = (void *) func;
-	}
+	/* Find or compile the function */
+	func = plpgsql_compile(fcinfo);
 
 	/*
 	 * Determine if called as function or trigger and call appropriate
 	 * subhandler
 	 */
-	if (isTrigger)
+	if (CALLED_AS_TRIGGER(fcinfo))
 		retval = PointerGetDatum(plpgsql_exec_trigger(func,
 									   (TriggerData *) fcinfo->context));
 	else
@@ -143,34 +127,7 @@ plpgsql_call_handler(PG_FUNCTION_ARGS)
 	 * Disconnect from SPI manager
 	 */
 	if (SPI_finish() != SPI_OK_FINISH)
-		elog(ERROR, "plpgsql: SPI_finish() failed");
+		elog(ERROR, "SPI_finish failed");
 
 	return retval;
 }
-
-
-/*
- * Check to see if a compiled function is still up-to-date.  This
- * is needed because CREATE OR REPLACE FUNCTION can modify the
- * function's pg_proc entry without changing its OID.
- */
-static bool
-func_up_to_date(PLpgSQL_function * func)
-{
-	HeapTuple	procTup;
-	bool		result;
-
-	procTup = SearchSysCache(PROCOID,
-							 ObjectIdGetDatum(func->fn_oid),
-							 0, 0, 0);
-	if (!HeapTupleIsValid(procTup))
-		elog(ERROR, "plpgsql: cache lookup for proc %u failed",
-			 func->fn_oid);
-
-	result = (func->fn_xmin == procTup->t_data->t_xmin &&
-			  func->fn_cmin == procTup->t_data->t_cmin);
-
-	ReleaseSysCache(procTup);
-
-	return result;
-}
